#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: structure
#------------------------------------------------------------------------------
# Module to build the working structure 
# (in the form of an RDF graph grouping the different structures needed)
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

import glob
import logging
from rdflib import Graph
#import time, datetime


#==============================================================================
# Parameters
#==============================================================================

# Logging
logger = logging.getLogger(__name__)


#==============================================================================
# Graph Loading
#==============================================================================
    
def load_config(config, work_graph):
    logger.debug("----- Configuration Loading")
    
    work_graph.parse(config.schema_file)
    logger.debug("-------- RDF Schema ({0})".format(len(work_graph)))
    
    work_graph.parse(config.semantic_net_file)
    logger.debug("-------- Semantic Net Definition ({0})".format(len(work_graph)))
        
    work_graph.parse(config.config_param_file)
    logger.debug("-------- Config Parameter Definition ({0})".format(len(work_graph)))


def load_frame(config, work_graph):
    logger.debug("----- Frame Ontology Loading")
        
    if config.is_base_ontology_produced_as_output():
        work_graph.parse(config.base_ontology_file)
        logger.debug("-------- Base Ontology produced as output ({0})".format(len(work_graph)))
    else:
        work_graph.parse(config.frame_ontology_file)
        logger.debug("-------- System Frame Ontology (" + str(len(work_graph)) + ")")
        work_graph.parse(config.frame_ontology_seed_file)
        logger.debug("-------- System Frame Ontology Seed (" + str(len(work_graph)) + ")")
    
    
def load_sentence(config, work_graph, sentence_file):
    logger.debug("----- Sentence Loading")
    work_graph.parse(sentence_file)
    logger.debug("-------- {0} ({1})".format(sentence_file, len(work_graph))) 
    
    
def load_all_sentences(config, work_graph):
    logger.debug("----- Sentences Loading")
    
    sentence_count = 0
    for file_ref in glob.glob(config.source_sentence_file, recursive = True):
        sentence_count += 1
        work_graph.parse(file_ref)
        # logger.debug("----- " + file_ref + " (" + str(len(work_graph)) + ")") 
        
    logger.debug("-------- Loaded sentence number: " + str(sentence_count))
    

#==============================================================================
# Result (export)
#==============================================================================
     
def export_result(config, work_graph, export_file):    
    work_graph.serialize(destination=export_file, 
                         base=config.base_uri + '/' + config.uuid_str, 
                         format='turtle')
    
    
def finalize_export_file(config, export_file):
    """ finalize the export file with some necessary adjustments """
    
    with open(export_file, "rt") as file:
        x = file.read()
    
    # -- old ---
    # with open(export_file, "wt") as file:
    #     x = x.replace(
    #         "@prefix sys: <https://tenet.tetras-libre.fr/frame/system-ontology/> .",
    #         """
    #         @prefix sys: <https://tenet.tetras-libre.fr/frame/system-ontology/> .
    #         @prefix sys-Event: <https://tenet.tetras-libre.fr/frame/system-ontology/Event#> .
    #         @prefix sys-State_Property: <https://tenet.tetras-libre.fr/frame/system-ontology/State_Property#> .
    #         @prefix sys-abstract_thing: <https://tenet.tetras-libre.fr/frame/system-ontology/abstract_thing#> .
    #         @prefix sys-action_verb: <https://tenet.tetras-libre.fr/frame/system-ontology/action_verb#> .
    #         @prefix sys-agent: <https://tenet.tetras-libre.fr/frame/system-ontology/agent#> .
    #         @prefix sys-attributive_verb: <https://tenet.tetras-libre.fr/frame/system-ontology/attributive_verb#> .
    #         @prefix sys-component: <https://tenet.tetras-libre.fr/frame/system-ontology/component#> .
    #         @prefix sys-message: <https://tenet.tetras-libre.fr/frame/system-ontology/message#> .
    #         @prefix sys-place: <https://tenet.tetras-libre.fr/frame/system-ontology/place#> .
    #         @prefix owl: <http://www.w3.org/2002/07/owl#> .
    #         @prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .
    #         @prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
    #         @prefix sys-relation: <https://tenet.tetras-libre.fr/frame/system-ontology/relation/> .
    #         @prefix xsd: <http://www.w3.org/2001/XMLSchema#> .
    #         """)
    #     file.write(x)
    
    # -- update system-ontology namespace
    with open(export_file, "wt") as file:
        orig_sys_namespace = "https://tenet.tetras-libre.fr/system-ontology/"
        new_sys_namespace = config.output_ontology_namespace
        x = x.replace(orig_sys_namespace, new_sys_namespace)
        file.write(x)
        
    # -- reloading of work_graph after change
    work_graph = Graph()
    work_graph.parse(export_file)
    
    return work_graph
   
    
def obtain_graph_reference(work_graph):
    """ obtain references (id, sentence) of the work graph """
    
    graphId = '-'
    graphSentence = '-'
        
    select_query = """
        PREFIX ns3: <http://amr.isi.edu/rdf/core-amr#>

        SELECT DISTINCT ?graphId ?graphSentence
        WHERE {
            ?r a ns3:AMR ;
                ns3:has-id ?graphId ;
                ns3:has-sentence ?graphSentence.
        }"""

    qres = work_graph.query(select_query)
    for row in qres:
        graphId = f'{row.graphId}'
        graphSentence = f'{row.graphSentence}'

    return graphId, graphSentence

     
#==============================================================================
# Main Function
#==============================================================================
  
def prepare_work_graph(config, sentence_file):
    """ create working structure as RDF graph"""
    
    try:
        logger.info("-- Work Structure Preparation")
        
        # -- Graph Initialization
        logger.debug("--- Graph Initialization")
        work_graph = Graph()
        load_config(config, work_graph)
        load_frame(config, work_graph) 
        
        # -- Source Data Import
        logger.debug("--- Source Data Import") 
        if config.process_level == 'sentence':
            load_sentence(config, work_graph, sentence_file)
        else: # process_level == 'document'
            load_all_sentences(config, work_graph)
    
        # -- Result
        if config.technical_dir_path is not None:
            logger.debug('--- Export work graph as turtle')
            export_file = config.output_file
            logger.debug('----- Work graph file: {0} '.format(export_file))
            export_result(config, work_graph, export_file)
            work_graph = finalize_export_file(config, export_file)
        
        # -- Information logging about structure
        graphId, graphSentence = obtain_graph_reference(work_graph)
        logger.info(f"----- Sentence (id): {graphId}")
        logger.info(f"----- Sentence (text): {graphSentence}")
        
        return work_graph
    
    except:
        logger.error("!!! An exception occurred !!!") 
  
    
def prepare_sentence_work(config, sentence_file):
    """ create working structure at sentence level"""
    config.process_level = 'sentence'
    return prepare_work_graph(config, sentence_file)
        
  
def prepare_document_work(config):
    """ create working structure at document level"""
    config.process_level = 'document'
    return prepare_work_graph(config, ' *** document *** ')
