#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Net
#------------------------------------------------------------------------------
# Class to handle semantic nets 
#==============================================================================

import rdflib
from rdflib import URIRef, Literal, BNode
from rdflib.namespace import FOAF, RDF
from rdflib.namespace import NamespaceManager
from rdflib.term import _is_valid_uri

from febTransduction.net import SemanticNetReferenceHandle
from febTransduction.rdfterm_computer import produce_uriref
from febTransduction.query_builder import generate_select_query


#==============================================================================
# Class
#==============================================================================

class Net:
    """ Class to handle semantic net.
    """    
    
    #--------------------------------------------------------------------------
    # Constructor
    #--------------------------------------------------------------------------
              
    def __init__(self, support_graph, uri=None):
        
        # -- Net Support
        self.support_graph = support_graph
        self.referenceHandle = SemanticNetReferenceHandle()
        
        # -- Net Type
        self.type_name = 'default'
        self.type_id = 'Net'
        self.type_uri = produce_uriref(self.support_graph, f'net:{self.type_id}')
        
        # -- Net URI
        self._uri = produce_uriref(self.support_graph, uri)        
           
        # -- Net Attributes
        self.attr_list = ['node', 'base_node', 'structure', 'naming']
        self._node = []
        self._node_list = []
        self._base_node = None
        self._structure = None
        self._naming = None
  
        
  
    #--------------------------------------------------------------------------
    # Accessor Base Methods
    #--------------------------------------------------------------------------
    
    def __find_attr_value_in_graph(self, attr_key):
        
        # -- query code
        select_data_list = ['?attr_value']
        clause_list = []
        predicate = self.referenceHandle.get_predicate_uri(f'{attr_key}')
        clause_list.append(f'{self._uri} {predicate} ?attr_value.')
        query_code = generate_select_query(select_data_list, clause_list)
        
        # -- query application
        query_result = self.support_graph.query(query_code)
                
        return query_result
    
    def __select_value_in_graph(self, attr_key):        
        res = None
        value_set = self.__find_attr_value_in_graph(attr_key)
        assert len(value_set) <= 1
        for v in value_set:
            res = v.attr_value              
        return res
    
    def __select_value_list_in_graph(self, attr_key):        
        res = []
        value_set = self.__find_attr_value_in_graph(attr_key)
        for v in value_set:
            res.append(v.attr_value)          
        return res

    def get_attribute_value(self, **attr):
        
        res = None
        
        for attr_key, attr_value in attr.items():
            if (self.uri is not None) & (attr_value is None):
                res = self.__select_value_in_graph(attr_key)
            elif (self.uri is not None) & (attr_value == []):
                res = self.__select_value_list_in_graph(attr_key)
            else:
                res = attr_value
                
        return res 

    def set_attribute_value(self, attr_key, attr_value):
        if isinstance(attr_key, list):
            if isinstance(attr_value, list):
                attr_key += attr_value
            else:
                attr_key.append(attr_value)
        else:
            attr_key = attr_value
        return attr_key
    

    #--------------------------------------------------------------------------
    # Accessors for Net URI
    #--------------------------------------------------------------------------
        
    @property
    def uri(self):
        return self._uri       
    
    @uri.setter
    def uri(self, new_value):
        self._uri = produce_uriref(self.support_graph, new_value) 
    
        
    #--------------------------------------------------------------------------
    # Accessors for Net Attributes
    #--------------------------------------------------------------------------
        
    @property
    def node_list(self):
        self._node = self.get_attribute_value(node=self._node)
        return self._node
    
    @node_list.setter
    def node_list(self, new_value):
        assert isinstance(new_value, list), f'the new value must be a list {new_value}'
        self._node = []
        for value in new_value:
            value = produce_uriref(value)
            self_node.append(value)
        
        
    @property
    def node(self):
        self._node = self.get_attribute_value(base_node=self._node)
        return self._base_node
    
    @node.setter
    def node(self, new_value):
        self._node = self.set_attribute_value(self._node, new_value)
        
        
    @property
    def base_node(self):
        self._base_node = self.get_attribute_value(base_node=self._base_node)
        return self._base_node
    
    @base_node.setter
    def base_node(self, new_value):
        self._base_node = self.set_attribute_value(self._base_node, new_value)
        
        
    @property
    def structure(self):
        self._structure = self.get_attribute_value(structure=self._structure)
        return self._structure
    
    @structure.setter
    def structure(self, new_value):
        self._structure = self.set_attribute_value(self._structure, new_value)
        
    @property
    def naming(self):
        self._naming = self.get_attribute_value(naming=self._naming)
        return self._naming
    
    @naming.setter
    def naming(self, new_value):
        self._naming = self.set_attribute_value(self._naming, new_value)
        
        
        
    #--------------------------------------------------------------------------
    # Composition Method
    #--------------------------------------------------------------------------
    
    def compose(self, *nets):
        
        assert len(nets) > 0, 'at least one net is needed to compose a new net'
        
        # -- Base Node and Structure Setting
        self.base_node = nets[0].base_node
        self.structure = nets[0].structure
        
        # -- Node(s) Setting
        for n in nets:
            self.node = n.node
        
        
        
    #--------------------------------------------------------------------------
    # Consolidation Method (finalize)
    #--------------------------------------------------------------------------
    
    def __consolidate_uri(self):
        if self.uri is None:
            if self.naming is not None:
                self.uri = URIRef(f'net:{self.type_name}_{self.naming}')
    
    def finalize(self):
        self.__consolidate_uri()
        
        
        
    #--------------------------------------------------------------------------
    # Triple Generation Method
    #--------------------------------------------------------------------------
    
    def __define_rdf_term(self, element):
        term = None
        if isinstance(value, rdflib.term.URIRef):
            value = value.n3(self.support_graph.namespace_manager)
            triple = f'{uri} {predicate} {value}.'
        elif isinstance(value, str):
            triple = f'{uri} {predicate} "{value}".'
    
    
    def __define_triple(self, uri, predicate, value):
        
        triple = None
        
        if isinstance(value, rdflib.term.URIRef):
            value = value.n3(self.support_graph.namespace_manager)
            triple = f'{uri} {predicate} {value}.'
        elif isinstance(value, str):
            triple = f'{uri} {predicate} "{value}".'
            
        return triple
        
    
    def __define_triple_list(self, uri, attr_predicate, attr_value):
        
        triples = []
        
        if (isinstance(attr_value, rdflib.term.URIRef) or 
            isinstance(attr_value, str)):
            new_triple = self.__define_triple(uri, attr_predicate, attr_value)
            if new_triple is not None: triples.append(new_triple)
            
        elif isinstance(attr_value, list):
            for value in attr_value:
                new_triple = self.__define_triple(uri, attr_predicate, value)
                if new_triple is not None: triples.append(new_triple)
            
        return triples
    
    
    def __generate_attribute_triples(self, uri):
        assert (self.uri is not None), 'URI must be set'
        
        triple_definition = []
        
        for attr in self.attr_list:
            attr_predicate = self.referenceHandle.get_predicate_uri(attr)
            attr_value = eval(f'self._{attr}')
            triple_definition += self.__define_triple_list(uri, attr_predicate, attr_value)
        
        return triple_definition
    
    
    def generate_triple_definition(self):
        
        triple_definition = []
        
        uri = self.uri
        if uri is not None:
            type_uriref = produce_uriref(self.support_graph, self.type_uri)
            triple_definition.append((self.uri, RDF.type, type_uriref))
            #triple_definition += self.__generate_attribute_triples(uri)
            
        return triple_definition