#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: SPARQL Query Builder
#------------------------------------------------------------------------------
# Class to define a Compositional Transduction Rule (CTR), namely a rule 
# associating a semantic graph pattern and a semantic net constructor.  
# Several methods are proposed to translate the rule into a sequence of SPARQL 
# queries applicable to a semantic graph.
#==============================================================================

# -- Useful Constant(s)
DEFAULT_ATTRIBUTE_VALUE = f'\"NA\"'
INDENT_STR = '            '
SENTENCE_REF = '?sentenceRef'


#==============================================================================
# Method to generate construct query
#==============================================================================

prefix_list = [('owl', '<http://www.w3.org/2002/07/owl#>'),
               ('rdf', '<http://www.w3.org/1999/02/22-rdf-syntax-ns#>'),
               ('rdfs', '<http://www.w3.org/2000/01/rdf-schema#>'),
               ('xsd', '<http://www.w3.org/2001/XMLSchema#>'),
               ('amr', '<https://amr.tetras-libre.fr/rdf/schema#>'),
               ('ns1', '<http://amr.isi.edu/frames/ld/v1.2.2/>'),
               ('ns2', '<http://amr.isi.edu/rdf/amr-terms#>'),
               ('ns3', '<http://amr.isi.edu/rdf/core-amr#>'),
               ('ns4', '<http://amr.isi.edu/entity-types#>'),
               ('net', '<https://tenet.tetras-libre.fr/semantic-net#>'),
               ('cprm', '<https://tenet.tetras-libre.fr/config/parameters#>'),
               ('fprm', '<https://tenet.tetras-libre.fr/frame/parameters#>'),
               ('base-out', '<https://tenet.tetras-libre.fr/base-ontology#>'),
               ('ext-out', '<https://tenet.tetras-libre.fr/extract-result#>')]


def get_prefix_def():
    prefix_def = ''
    for (ident, iri) in prefix_list:
        prefix_def += f'PREFIX {ident}: {iri}\n'
    return prefix_def


def generate_construct_query(construct_part, clause_part, binding_part=''):
    
    prefix = get_prefix_def()
    
    query_code = f'{prefix}'
    query_code += f'\n CONSTRUCT {{ {construct_part} \n }}'
    query_code += f'\n WHERE {{ {clause_part} \n {binding_part} \n }}'
    
    return query_code   


def generate_select_query(select_data_list, clause_list):
    
    prefix = get_prefix_def()
    
    first = True
    select_data_part = ''
    for data in select_data_list:
        if not first: select_data_part += ' '
        select_data_part += f'{data}' 
        first = False
    
    clause_part = ''
    for clause in clause_list:
        clause_part += f'\n    {clause}'
    
    query_code = f'{prefix}'
    query_code += f'\n SELECT {select_data_part}'
    query_code += f'\n WHERE {{ {clause_part} \n }}'
    
    return query_code    


def generate_insert_query(triple_list):
    
    prefix = get_prefix_def()
    
    data_part = ''
    for triple in triple_list:
        data_part += f'\n    {triple}'
    
    query_code = f'{prefix}'
    query_code += f'\n INSERT DATA {{ {data_part} \n }}'
    
    return query_code    
            
    
#==============================================================================
# Development Test
#==============================================================================
    
if __name__ == '__main__':
    
    print('\n' + ' *** Development Test ***')
    
    print('\n -- Generate SPARQL construct query')
    query = generate_construct_query('construct', 'identify', 'BIND')
    print(query)
    
    print('\n -- Generate SPARQL select query')
    select_data_list = ['net1', 'net2', 'net3']
    clause_list = ['net1 a net:ClassNet.', 
                   'net2 a net:PropertyNet.',
                   'net3 a net:ClassNet.']
    query = generate_select_query(select_data_list, clause_list)
    print(query)
    
    print('\n -- Generate SPARQL insert query')
    triple_list = ['net:net1 a net:ClassNet.', 
                   'net:net2 a net:PropertyNet.',
                   'net:net3 a net:ClassNet.']
    query = generate_insert_query(triple_list)
    print(query)
        
    print('\n' + ' *** - ***')