#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Transduction Rule
#------------------------------------------------------------------------------
# Class to define a Compositional Transduction Rule (CTR), namely a rule 
# associating a semantic graph pattern and a semantic net constructor.  
# Several methods are proposed to translate the rule into a sequence of SPARQL 
# queries applicable to a semantic graph.
#==============================================================================

if __name__ == '__main__':
    import os, sys
    LIB_PATH = f'{os.path.dirname(os.path.abspath(__file__))}/..'
    sys.path.insert(0, os.path.abspath(LIB_PATH))
    print(sys.path[0])

import logging

import janTransduction.query_builder as query_builder
import janTransduction.net as net

# -- Useful Constant(s)
DEFAULT_ATTRIBUTE_VALUE = f'\"NA\"'
INDENT_STR = '            '
SENTENCE_REF = '?sentenceRef'


#==============================================================================
# Useful Method
#==============================================================================

prefix_list = [('owl', '<http://www.w3.org/2002/07/owl#>'),
               ('rdf', '<http://www.w3.org/1999/02/22-rdf-syntax-ns#>'),
               ('rdfs', '<http://www.w3.org/2000/01/rdf-schema#>'),
               ('xsd', '<http://www.w3.org/2001/XMLSchema#>'),
               ('amr', '<https://amr.tetras-libre.fr/rdf/schema#>'),
               ('ns1', '<http://amr.isi.edu/frames/ld/v1.2.2/>'),
               ('ns2', '<http://amr.isi.edu/rdf/amr-terms#>'),
               ('ns3', '<http://amr.isi.edu/rdf/core-amr#>'),
               ('ns4', '<http://amr.isi.edu/entity-types#>'),
               ('net', '<https://tenet.tetras-libre.fr/semantic-net#>'),
               ('cprm', '<https://tenet.tetras-libre.fr/config/parameters#>'),
               ('fprm', '<https://tenet.tetras-libre.fr/frame/parameters#>'),
               ('base-out', '<https://tenet.tetras-libre.fr/base-ontology#>'),
               ('ext-out', '<https://tenet.tetras-libre.fr/extract-result#>')]

def get_prefix_def():
    prefix_def = ''
    for (ident, iri) in self.prefix_list:
        prefix_def += f'PREFIX {ident}: {iri}\n'
    return prefix_def


def generate_query(prefix, construct_part, select_part, binding_part):
    prefix = get_prefix_def()
    return f"""{prefix}
CONSTRUCT {{
    {construct_part}
}}
WHERE {{
    {select_part}
    {binding_part}
}}
"""

#==============================================================================
# Net Class
#==============================================================================

class Rule:
    """ Class to define a Compositional Transduction Rule (CTR), namely a rule 
        associating a semantic graph pattern and a semantic net constructor.  
        Several methods are proposed to translate the rule into a sequence of 
        SPARQL queries applicable to a semantic graph.
    """
    
    #--------------------------------------------------------------------------
    # Class Attributes
    #--------------------------------------------------------------------------

    # -- Operation Reference 
    INITIALIZED = f'net:initialized'
    RELATION_PROPAGATED = f'net:relation_propagated'
    RESTRICTION_ADDED = f'net:restriction_added'
    FINALIZED = f'net:finalized'
    
    #--------------------------------------------------------------------------
    # Constructor
    #--------------------------------------------------------------------------
              
    def __init__(self, label, net):
        
        # -- Base
        self.label = label
        self.new_net = net        
        
        # -- Pattern
        self._identification_pattern_dict = {}
        self._composition_pattern_list = []
        self._restriction_pattern_list = []
        
        # -- SPARQL Query
        self.query_list = []
        
    
    #--------------------------------------------------------------------------
    # Accessor(s)
    #--------------------------------------------------------------------------
        
    # --

    
    #--------------------------------------------------------------------------
    # Method(s) to add pattern
    #--------------------------------------------------------------------------

    def add_identification_pattern(self, net, **net_attribute):
        self._identification_pattern_dict[net] = net.identify_attribute(**net_attribute)


    def add_composition_pattern(self, net_1, relation, net_2):
        self._composition_pattern_list.append((net_1, relation, net_2))


    def add_restriction_pattern(self, net_1, relation, net_2):
        self._restriction_pattern_list.append((net_1, relation, net_2))
        
    
    #--------------------------------------------------------------------------
    # Method(s) to construct a new semantic net by composition of net(s)
    #--------------------------------------------------------------------------
    
    def __define_identification_clause(self, *nets):
        clause_part = f"\n{INDENT_STR}"
        clause_part += f"# -- Identify Net(s)"
        for net in nets:
            clause_part += f"\n{INDENT_STR}"
            clause_part += net.identify()
            if net in self._identification_pattern_dict:
                clause_part += self._identification_pattern_dict[net]                
        return clause_part
    
    
    def __define_pattern_clause(self):
        clause_part = f"\n{INDENT_STR}"
        clause_part += f"# -- Identify Pattern "
        for (net_1, relation, net_2) in self._composition_pattern_list:
            clause_part += f"\n{INDENT_STR}"
            clause_part += f'{net_1.id} {relation} {net_2.id}.'
        return clause_part
    
    
    def __define_tracking_data_selection_clause(self, *nets):
        clause_part = f"\n{INDENT_STR}"
        clause_part += f"# -- Data Selection "
        for net in nets:
            clause_part += f"\n{INDENT_STR}"
            clause_part += net.select_tracking_data()
        return clause_part
    
    
    def __define_new_net_binding_clause(self, *nets):
        
        assert len(nets) > 0, '__define_new_net_binding_clause impossible without net'
        
        clause_part = f"\n{INDENT_STR}"
        clause_part += f"""# *** Identify variable label of base leaf ***
            {nets[0].select_data('base_node')}
            {nets[0].base_node} a amr:AMR_Leaf ;
                amr:hasVariable ?variable.
                ?variable amr:label ?varLabel."""
                
        clause_part += f"\n\n{INDENT_STR}"
        clause_part += f"# -- Naming Binding \n{INDENT_STR}"
        clause_part += self.new_net.bind_naming(*nets)
        
        clause_part += f"\n\n{INDENT_STR}"
        clause_part += f"# -- URI Binding"
        clause_part += self.new_net.bind_uri(self.new_net.naming, '?varLabel')
        
        return clause_part
        
        
    def __define_new_net(self):
        construct_part = f"\n{INDENT_STR}"
        construct_part += f'# -- New net \n'
        construct_part += self.new_net.define()
        return construct_part
        
        
    def __define_composition(self, *nets):
        construct_part = f"\n{INDENT_STR}"
        construct_part += f'# -- Composition Tracking for {self.new_net.id}'
        main = True
        for net in nets:
            construct_part += f"\n{INDENT_STR}"
            construct_part += self.new_net.track_composante(net, main)
            main = False
        return construct_part
        
    def __define_deprecate_net(self, *nets):
        if (nets is not None) & (len(nets) != 0):
            construct_part = f'\n{INDENT_STR}'
            construct_part += f'# -- Deprecate net \n{INDENT_STR}'
            construct_part += f'{nets[0].id} a net:Deprecated_Net.'
        return construct_part
        
        
    def compose(self, *nets):
        
        query_label = 'new net construction'
        
        # -- Clause Part
        clause_part = ''
        clause_part += self.__define_identification_clause(*nets)
        clause_part += '\n'
        clause_part += self.__define_pattern_clause()
        clause_part += '\n'
        clause_part += self.__define_tracking_data_selection_clause(*nets)
        clause_part += '\n'
        clause_part += self.__define_new_net_binding_clause(*nets)
        
        # -- Construct Part
        construct_part = ''
        construct_part += self.__define_new_net()
        construct_part += self.__define_composition(*nets)
        construct_part += self.__define_deprecate_net(*nets)
        
        # -- Query Generation
        compose_query = query_builder.generate_construct_query(construct_part, clause_part)
        self.query_list.append((query_label, compose_query))
        
    
    #--------------------------------------------------------------------------
    # Method(s) to add data in a new semantic net
    #--------------------------------------------------------------------------
          
    def __select_data_value(self, data_value): # TODO
        clause_part = ''
        return clause_part
        
    def __add_data(self, key, value): # TODO
        construct_part = ''
        return construct_part
    
    def compute_data(self, **data):
        
        query_label = 'net data computing'
        
        # -- Clause Part
        clause_part = ''
        clause_part += self.new_net.identify(step_num=1)
        for data_value in data.values():
            clause_part += __select_data_value(self, data_value)
        
        # -- Construct Part
        construct_part = ''
        for data_key, data_value in data.items():
            construct_part += self.__add_data(data_key, data_value)
        #construct_part += self.new_net.track_progression(step_num=1)
        
        # -- Query Generation
        compose_query = query_builder.generate_construct_query(construct_part, clause_part)
        self.query_list.append((query_label, compose_query))
        
    
    
    
    
#==============================================================================
# Development Test
#==============================================================================
    
if __name__ == '__main__':
    
    print('\n' + ' *** Development Test ***')  
    
    # print('\n' + ' -- Net Definition')
    property_net_0 = net.PropertyNet(0)
    property_net_1 = net.PropertyNet(1)
    phenomena_net = net.PhenomenaNet()
    or_composite_class_net = net.OrCompositeClassNet()
    
    # print('\n' + ' -- Test Rule 1')
    rule = Rule('Test Rule', or_composite_class_net)
    rule.add_identification_pattern(phenomena_net, phenomena_type='amr:phenomena_conjunction_or')
    rule.add_composition_pattern(property_net_1, 'amr:role_ARG0', property_net_0)
    rule.add_composition_pattern(property_net_1, 'amr:role_ARG1', phenomena_net)
    rule.compose(property_net_0, property_net_1, phenomena_net)
    
    rule.compute_data()
    
    num = 0
    for query_label, query in rule.query_list:
        num += 1
        print(f'*** query {num} - {query_label} ***\n{query}\n')
    
    # print('\n' + ' -- load a rule definition from a dictionary')
    # rule.load_dict(rule_def_1)
    # print(rule)
    
    # print('\n' + ' -- load a prefix list')
    # rule.load_prefix_list(prefix_list_1)
    # print(rule)
    
    # print('\n' + ' -- get the SPARQL query')
    # query = rule.get_query()
    # print(query)
        
    print('\n' + ' *** - ***')