#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Logical Set Net Query Builder
#------------------------------------------------------------------------------
# Class to generate SPARQL query parts related to semantic nets 
#==============================================================================

#==============================================================================
# Importing required modules
#==============================================================================

from .net import Net


#==============================================================================
# Net Class
#==============================================================================

class LogicalSetNet(Net):
    """ Class to generate SPARQL query parts related to semantic nets.
    """
    
    #--------------------------------------------------------------------------
    # Constructor(s)
    #--------------------------------------------------------------------------
           
    def __init__(self, num=''):
        
        # -- Net Signature
        self.type_name = 'logicalSet'
        self.type_id = 'Logical_Set_Net'
        self.id = f'?{self.type_name}Net{num}'
        self.type_uri = f'net:{self.type_id}'
        
        # -- Parent init
        super().__init__(signature=True)
        
        # -- Net Attributes
        self.logical_constraint = f'{self.id}Restriction'
        self.property_net = f'{self.id}PropertyNet'
        self.content_net_1 = f'{self.id}ContentNet1'
        self.content_net_2 = f'{self.id}ContentNet2'
        self.content_net = f'{self.id}ContentNet'
        self.naming = f'{self.id}Naming'
        self.restriction = f'{self.id}Restriction'
        
        self.predicate_table.update({
            'logical_constraint': 'hasLogicalConstraint',
            'property_net': 'bindPropertyNet',
            'content_net_1': 'containsNet1',
            'content_net_2': 'containsNet2',
            'content_net': 'containsNet',
            'naming': 'hasNaming',
            'restriction': 'bindRestriction'
            
            })
        

    #--------------------------------------------------------------------------
    # Private data accessor(s)
    #--------------------------------------------------------------------------
        
    def __get_predicate(self, attribute_reference):
        predicate_reference = self.predicate_table[f'{attribute_reference}']
        return f'net:{predicate_reference}'
     
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Construct' parts  
    #-------------------------------------------------------------------------- 
    
    def define_naming(self):
        predicate = self.__get_predicate('naming')
        return f"""
            # -- Naming Definition
            {self.id} {predicate} {self.naming}."""
            
        
    #--------------------------------------------------------------------------
    # Method(s) to build 'Clause' parts  
    #--------------------------------------------------------------------------
    
    def identify_content_net_1(self, target_id):
        predicate = self.__get_predicate('content_net_1')
        return f"""
            # -- Identify content net 1
            {self.id} {predicate} {target_id}.""" 
            
    
    def identify_content_net_2(self, target_id):
        predicate = self.__get_predicate('content_net_2')
        return f"""
            # -- Identify content net 2
            {self.id} {predicate} {target_id}.""" 
    
    
    def identify_content_net(self, target_id):
        predicate = self.__get_predicate('content_net')
        return f"""
            # -- Identify content net
            {self.id} {predicate} {target_id}.""" 
            
    
    def identify_disjoint_content_net(self, target_id_1, target_id_2):
        predicate1 = self.__get_predicate('content_net_1')
        predicate2 = self.__get_predicate('content_net_2')
        return f"""
            # -- Identify disjoint content net
            {self.id} {predicate} {target_id_1}.
            {self.id} {predicate} {target_id_2}.
            FILTER ( {target_id_1} != {target_id_2} ).""" 
            
    def identify_content_net_number(self, number_ref):
        predicate = self.__get_predicate('content_net')
        return f"""
            {{
                # -- Identify content net number
                SELECT (COUNT(?contentNet) AS {number_ref})
            		WHERE {{
                        {self.id} {predicate} ?contentNet.
                }}
            }}""" 
            
    
    def identify_restriction(self, target_id):
        predicate = self.__get_predicate('restriction')
        return f"""
            # -- Identify restriction
            {self.id} {predicate} {target_id}.""" 
    
    
    #--------------------------------------------------------------------------
    # Method(s) to build 'Binding' parts  
    #--------------------------------------------------------------------------
    
    def bind_naming(self, property_ref, content_1_ref, content_2_ref):
            
        ref1 = f"{self.id}Ref1"
        ref2 = f"{self.id}Ref2"
        # -- ref3 = f"{self.id}Ref3"
        
        return f"""    
            # -- New Naming
            {self.identify_content_net_number('?contentNumber')}
            BIND (CONCAT({property_ref}, '-', {content_1_ref}) AS {ref1}).
            BIND (CONCAT({ref1}, '-and-', {content_2_ref}) AS {ref2}).
            BIND (IF (?contentNumber > 2, CONCAT({ref2}, '-etc'), {ref2}) AS {self.naming})."""