#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: AMR CTR at 'Data Preprocessing' level
#------------------------------------------------------------------------------
# Module grouping compositional transduction rule_sets (CTR) for the analysis of 
# AMR structures, at 'Data Preprocessing' level
#==============================================================================

#==============================================================================
# CTR Set (CTR as Sparql querries)
#==============================================================================

rule_set = {}


# ---------------------------------------------
# Preprocessing: Reclassification of concepts
# ---------------------------------------------

rule_set['reclassify-concept-1'] = {
    'label': "reclassify-concept-1",
    'comment': "Reclassify terms concept to AMR Special concept",
    'construction': """
    ?c rdfs:subClassOf amr:AMR_Special_Concept ;
        amr:hasPhenomenaLink ?phenomena.
    """,
    'clause': """
    ?c a ?amrConceptClass.
        VALUES ?amrConceptClass {ns3:Frame ns3:Concept ns3:NamedEntity}
    ?phenomena rdfs:subClassOf* amr:AMR_Phenomena ;
        amr:hasConceptLink ?phenomenaLink. 
    FILTER (?cLabel1 = ?phenomenaLink ||
            ?cLabel2 = ?phenomenaLink).
    """,
    'binding':"""
    BIND (strafter(str(?c), str(ns1:)) AS ?cLabel1).
    BIND (strafter(str(?c), '#') AS ?cLabel2).
    """
    }


rule_set['reclassify-concept-2'] = {
    'label': "reclassify-concept-2",
    'comment': "Reclassify terms concept to AMR Term concept",
    'construction': """
        ?c rdfs:subClassOf amr:AMR_Term_Concept.
    """,
    'clause': """
        ?c a ns3:Concept.
        FILTER NOT EXISTS { ?c rdfs:subClassOf amr:AMR_Special_Concept. }
        FILTER ( CONTAINS(str(?c), str(ns2:)) ).
        FILTER ( ?c != ns3:Frame ).
        FILTER ( ?c != ns3:NamedEntity ).
    """,
    'binding': ''
    }

rule_set['reclassify-concept-3'] = {
    'label': "reclassify-concept-3",
    'comment': "Reclassify core concept to AMR Relation concept",
    'construction': """
        ?c rdfs:subClassOf amr:AMR_Relation_Concept.
    """,
    'clause': """
        ?c a ns3:Concept.
        FILTER NOT EXISTS { ?c rdfs:subClassOf amr:AMR_Special_Concept. }
        FILTER ( CONTAINS(str(?c), str(ns3:)) ).
        FILTER ( ?c != ns3:Frame ).
        FILTER ( ?c != ns3:NamedEntity ).
    """,
    'binding': ''
    }

rule_set['reclassify-concept-4'] = {
    'label': "reclassify-concept-4",
    'comment': "Reclassify named entity to AMR Term concept",
    'construction': """
        ?c rdfs:subClassOf amr:AMR_Term_Concept.
    """,
    'clause': """
        ?c a ns3:NamedEntity.
        FILTER NOT EXISTS { ?c rdfs:subClassOf amr:AMR_Special_Concept. }
    """,
    'binding': ''
    }

rule_set['reclassify-concept-5'] = {
    'label': "reclassify-concept-5",
    'comment': "Reclassify frame to AMR Predicat concept",
    'construction': """
        ?c rdfs:subClassOf amr:AMR_Predicat_Concept.
    """,
    'clause': """
        ?c a ns3:Frame.
        FILTER NOT EXISTS { ?c rdfs:subClassOf amr:AMR_Special_Concept. }
    """,
    'binding': ''
    }


# ---------------------------------------------
# Preprocessing: Reclassification of variables
# ---------------------------------------------

rule_set['reclassify-variable'] = {
    'label': "reclassify-variable",
    'comment': "Reclassify concept instance to AMR variable",
    'construction': """
        ?v a amr:AMR_Variable.
    """,
    'clause': """
        ?c rdfs:subClassOf* amr:AMR_Concept.
        ?v a ?c.
    """,
    'binding': ''
    }


# ---------------------------------------------
# Preprocessing: Reclassification of roles
# ---------------------------------------------

rule_set['reclassify-role-1'] = {
    'label': "reclassify-role-1",
    'comment': "Reclassify role to AMR core role",
    'construction': """
        ?newRole rdfs:subClassOf amr:AMR_Core_Role ;
            amr:fromAmrLkFramerole ?r.
    """,
    'clause': """
        ?r a ns1:FrameRole.
        """,
    'binding':"""
    BIND (strafter(str(?r), str(ns1:)) AS ?rLabel)
    BIND (strbefore(?rLabel, '.') AS ?rolesetId)
    BIND (strafter(?rLabel, '.') AS ?roleId)
	BIND (CONCAT(str(amr:), 'role') AS ?n1).
	BIND (CONCAT(?n1, '_', ?roleId) AS ?n2).
    BIND (uri(?n2) AS ?newRole).
    """
    }

rule_set['reclassify-role-2'] = {
    'label': "reclassify-rolet-2",
    'comment': "Reclassify role to AMR non-core role",
    'construction': """
        ?newRole rdfs:subClassOf amr:AMR_NonCore_Role ;
            amr:fromAmrLkRole ?r.
    """,
    'clause': """
        ?r a ns3:Role.
        FILTER ( ?r != ns1:FrameRole ).
        FILTER NOT EXISTS { ?newRole rdfs:subClassOf* amr:AMR_Role. }
        """,
    'binding':"""
    BIND (strafter(str(?r), '#') AS ?rLabel)
	BIND (CONCAT(str(amr:), 'role') AS ?n1).
	BIND (CONCAT(?n1, '_', ?rLabel) AS ?n2).
    BIND (uri(?n2) AS ?newRole).
    """
    }

rule_set['reclassify-role-3'] = {
    'label': "reclassify-rolet-3",
    'comment': "Add link with AmrLk role to AMR role",
    'construction': """
        ?newRole amr:fromAmrLkRole ?r.
    """,
    'clause': """
        ?r a ns3:Role.
        ?newRole rdfs:subClassOf* amr:AMR_Role.
        FILTER ( ?r != ns1:FrameRole ).
        """,
    'binding':"""
    BIND (strafter(str(?r), '#') AS ?rLabel)
	BIND (CONCAT(str(amr:), 'role') AS ?n1).
	BIND (CONCAT(?n1, '_', ?rLabel) AS ?n2).
    BIND (uri(?n2) AS ?newRole).
    """
    }


# ---------------------------------------------
# Preprocessing: Addition of leafs and edges
# ---------------------------------------------

rule_set['add-amr-leaf'] = {
    'label': "add-leaf-1",
    'comment': "Add AMR leaf corresponding to concept/variable relation",
    'construction': """
        ?newLeaf a amr:AMR_Leaf ;
            amr:hasVariable ?v ;
            amr:hasConcept ?c.
    """,
    'clause': """
        ?c rdfs:subClassOf* amr:AMR_Concept.
        ?v a ?c.
        """,
    'binding':"""
    BIND (strafter(str(?c), '#') AS ?cLabel1)
    BIND (strafter(str(?c), str(ns1:)) AS ?cLabel2)
    BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
    BIND (strafter(str(?v), '#') AS ?vLabel)
	BIND (CONCAT(str(amr:), 'leaf') AS ?n1).
	BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
	BIND (CONCAT(?n2, '_', ?vLabel) AS ?n3).
    BIND (uri(?n3) AS ?newLeaf).
    """
    }


rule_set['add-amr-edge-1'] = {
    'label': "add-amr-edge-1",
    'comment': ("Add AMR edge corresponding to a FrameRole relation",
                " between two leafs"),
    'construction': """
        ?newEdge a amr:AMR_Edge ;
            amr:hasRolesetID ?rolesetId ;
            amr:hasRoleID ?roleId ;
            amr:hasRoleTag ?roleTag.
		?leaf1 ?newEdge ?leaf2.
    """,
    'clause': """
        ?r a ns1:FrameRole.
        ?leaf1 amr:hasVariable ?v1.
        ?leaf2 amr:hasVariable ?v2.
		?v1 ?r ?v2.
        """,
    'binding':"""
    BIND (strafter(str(?r), str(ns1:)) AS ?rLabel)
    BIND (strbefore(?rLabel, '.') AS ?rolesetId)
    BIND (strafter(?rLabel, '.') AS ?roleId)
    BIND (?rLabel AS ?roleTag)
    BIND (strafter(str(?v1), '#') AS ?v1Label)
    BIND (strafter(str(?v2), '#') AS ?v2Label)
	BIND (CONCAT(str(amr:), 'edge') AS ?n1).
	BIND (CONCAT(?n1, '_', ?rLabel) AS ?n2).
	BIND (CONCAT(?n2, '_', ?v1Label) AS ?n3).
	BIND (CONCAT(?n3, '_', ?v2Label) AS ?n4).
    BIND (uri(?n4) AS ?newEdge).
    """
    }

rule_set['add-amr-edge-2'] = {
    'label': "add-amr-edge-2",
    'comment': ("Add AMR edge corresponding to a Role relation",
                " between two leafs"),
    'construction': """
        ?newEdge a amr:AMR_Edge ;
            amr:hasRolesetID ?rolesetId ;
            amr:hasRoleID ?roleId ;
            amr:hasRoleTag ?roleTag.
		?leaf1 ?newEdge ?leaf2.
    """,
    'clause': """
        ?r a ns3:Role.
        ?leaf1 amr:hasVariable ?v1.
        ?leaf2 amr:hasVariable ?v2.
		?v1 ?r ?v2.
        """,
    'binding':"""
    BIND (strafter(str(?r), '#') AS ?rLabel)
    BIND (strbefore(?rLabel, '.') AS ?rolesetId)
    BIND (strafter(?rLabel, '.') AS ?roleId)
    BIND (?rLabel AS ?roleTag)
    BIND (strafter(str(?v1), '#') AS ?v1Label)
    BIND (strafter(str(?v2), '#') AS ?v2Label)
	BIND (CONCAT(str(amr:), 'edge') AS ?n1).
	BIND (CONCAT(?n1, '_', ?rLabel) AS ?n2).
	BIND (CONCAT(?n2, '_', ?v1Label) AS ?n3).
	BIND (CONCAT(?n3, '_', ?v2Label) AS ?n4).
    BIND (uri(?n4) AS ?newEdge).
    """
    }


# ---------------------------------------------
# Preprocessing: Addition of root
# ---------------------------------------------

rule_set['add-amr-root'] = {
    'label': "add-amr-root",
    'comment': ("Add root instance in AMR_Root"),
    'construction': """
        ?newRoot a amr:AMR_Root ;
            amr:hasRootLeaf ?leaf ;
            amr:hasSentenceID ?graphId ;
            amr:hasSentenceStatement ?graphSentence ;
            amr:fromAmrLkRoot ?r.
    """,
    'clause': """
        ?r a ns3:AMR.
        ?r ns3:root ?lkRoot.
        ?r ns3:has-id ?graphId.
        ?r ns3:has-sentence ?graphSentence.
        ?leaf amr:hasVariable ?lkRoot.
		""",
    'binding':"""
    BIND (CONCAT(str(amr:), 'root') AS ?n1).
	BIND (CONCAT(?n1, '_', ?graphId) AS ?n2).
    BIND (uri(?n2) AS ?newRoot).
    """
    }


# ---------------------------------------------
# Preprocessing: Addition of values
# ---------------------------------------------

rule_set['add-amr-value-for-name-relation'] = {
    'label': "add-value-for-name-relation",
    'comment': ("Add a value for each name relation, corresponding to ",
                "variables with label in AMR Linked Data, ",
                "and to the name AMR relations in AMR graphs"),
    'construction': """
        ?newValue a amr:AMR_Value ;
            rdfs:label ?vLabel.
        ?v amr:name ?newValue.
    """,
    'clause': """
        ?v a amr:AMR_Variable.
        ?v rdfs:label ?vLabel.
		""",
    'binding':"""
    BIND (REPLACE(?vLabel, ' ', "") AS ?vRef).
	BIND (CONCAT(str(amr:), 'value') AS ?n1).
	BIND (CONCAT(?n1, '_', ?vRef) AS ?n2).
    BIND (uri(?n2) AS ?newValue).
    """
    }

rule_set['add-amr-value-for-quant-relation'] = {
    'label': "add-value-for-quant-relation",
    'comment': ("Add a value for each quant relation, corresponding to ",
                "variables with quant Property in AMR Linked Data, ",
                "and to the quant AMR relations in AMR graphs"),
    'construction': """
        ?newValue a amr:AMR_Value ;
            rdfs:label ?vQuant.
        ?v amr:quant ?newValue.
    """,
    'clause': """
        ?v a amr:AMR_Variable.
        ?v ns2:quant ?vQuant.
		""",
    'binding':"""
    BIND (REPLACE(?vQuant, ' ', "") AS ?vRef).
	BIND (CONCAT(str(amr:), 'value') AS ?n1).
	BIND (CONCAT(?n1, '_', ?vRef) AS ?n2).
    BIND (uri(?n2) AS ?newValue).
    """
    }


# ---------------------------------------------
# Preprocessing: Set labels for concepts, variables and roles
# ---------------------------------------------

rule_set['set-amr-label-for-concepts'] = {
    'label': "set-amr label-for-concepts",
    'comment': "Set AMR label for all concepts",
    'construction': """
        ?c amr:label ?cLabel.
    """,
    'clause': """
        ?c rdfs:subClassOf* amr:AMR_Concept.
        """,
    'binding':"""
    BIND (strafter(str(?c), '#') AS ?cLabel1)
    BIND (strafter(str(?c), str(ns1:)) AS ?cLabel2)
    BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
    """
    }

rule_set['set-amr-label-for-variables'] = {
    'label': "set-amr-label-for-variables",
    'comment': "Set AMR label for all variables",
    'construction': """
        ?v amr:label ?vLabel.
    """,
    'clause': """
        ?v a amr:AMR_Variable.
        """,
    'binding':"""
    BIND (strafter(str(?v), '#') AS ?vLabel)
    """
    }

rule_set['set-amr-label-for-roles'] = {
    'label': "set-amr-label-for-variables",
    'comment': "Set AMR label for all variables",
    'construction': """
        ?r amr:label ?rLabel.
    """,
    'clause': """
        ?r rdfs:subClassOf* amr:AMR_Role.
        """,
    'binding':"""
        BIND (strafter(str(?r), 'role_') AS ?rLabel)
    """
    }


# ---------------------------------------------
# Preprocessing: Update leafs and edges
# ---------------------------------------------

rule_set['update-amr-edge-role-1'] = {
    'label': "update-amr-edge-role-1",
    'comment': ("Update AMR edge by adding relation with AMR_Role",
                "(using comparison between label and role id)"),
    'construction': """
        ?edge amr:hasAmrRole ?role.
    """,
    'clause': """
        ?edge a amr:AMR_Edge ; 
            amr:hasRoleID ?edgeRoleId.
        ?role rdfs:subClassOf* amr:AMR_Role ;
            amr:label ?roleLabel.
        FILTER ( ?roleLabel != '' ).
        FILTER ( ?edgeRoleId = ?roleLabel ).
    """,
    'binding': ''
    }

rule_set['update-amr-edge-role-2'] = {
    'label': "update-amr-edge-role-2",
    'comment': ("Update AMR edge by adding relation with AMR_Role",
                "(using comparison between label and role tag)"),
    'construction': """
        ?edge amr:hasAmrRole ?role.
    """,
    'clause': """
        ?edge a amr:AMR_Edge ; 
            amr:hasRoleTag ?edgeRoleId.
        ?role rdfs:subClassOf* amr:AMR_Role ;
            amr:label ?roleLabel.
        FILTER ( ?roleLabel != '' ).
        FILTER ( ?edgeRoleId = ?roleLabel ).
    """,
    'binding': ''
    }

