#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: AMR CTR at 'Data Preprocessing' level
#------------------------------------------------------------------------------
# Module grouping compositional transduction rules (CTR) for the analysis of 
# AMR structures, at 'Data Preprocessing' level
#==============================================================================

#==============================================================================
# CTR Set (CTR as Sparql querries)
#==============================================================================

rule_set = {}


# ---------------------------------------------
# Reclassification of concepts
# ---------------------------------------------

rule_set['reclassify-concept-1'] = {
    'label': "reclassify-concept-1",
    'comment': ("Reclassify LK concept to AMR Relation concept for concept: ",
                " (1) corresponding to a LK term, frame or named entity,",
                " (2) corresponding to an AMR phenomena/relation."),
    'construction': """
        # -- New concept
        ?newConcept rdfs:subClassOf amr:AMR_Relation_Concept ;
            amr:label ?cLabel ;
            amr:hasPhenomenaLink ?phenomena ;
            amr:fromAmrLk ?c. 
            
        # -- Classification of the original concept as LinkedData (for tracing) 
        ?c rdfs:subClassOf amr:AMR_Linked_Data.
    """,
    'clause': """
        # -- Identify LK concept
        ?c a ?amrConceptClass.
            VALUES ?amrConceptClass {ns3:Frame ns3:Concept ns3:NamedEntity}
            
        # -- Identify phenomena matching
        ?phenomena rdfs:subClassOf* amr:AMR_Phenomena ;
            amr:hasConceptLink ?phenomenaLink.
        BIND (strafter(str(?c), str(ns1:)) AS ?cLabel1).
        BIND (strafter(str(?c), '#') AS ?cLabel2). 
        FILTER (?cLabel1 = ?phenomenaLink || ?cLabel2 = ?phenomenaLink).
        
        # -- Filter bugs
        FILTER NOT EXISTS { ?c rdfs:comment 'bug'. }
    """,
    'binding':"""
        # -- Label
        # -- already bound --- BIND (strafter(str(?c), '#') AS ?cLabel1)
        # -- already bound --- BIND (strafter(str(?c), str(ns1:)) AS ?cLabel2)
        BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
        
        # -- New concept
        BIND (CONCAT(str(amr:), 'concept') AS ?n1).
        BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
        BIND (uri(?n2) AS ?newConcept).
    """
    }


rule_set['reclassify-concept-2'] = {
    'label': "reclassify-concept-2",
    'comment': ("Reclassify LK concept to AMR Predicat concept for concept: ",
                " (1) corresponding to a predicat (ns3 concept),",
                " (2) not corresponding to a phenomena."),
    'construction': """
        # -- New concept
        ?newConcept rdfs:subClassOf amr:AMR_Predicat_Concept ;
            amr:label ?cLabel ;
            amr:fromAmrLk ?c. 
            
        # -- Classification of the original concept as LinkedData (for tracing) 
        ?c rdfs:subClassOf amr:AMR_Linked_Data.
    """,
    'clause': """
        # -- Identify LK relation concepts (ns3)
        ?c a ns3:Concept.
        FILTER ( CONTAINS(str(?c), str(ns3:)) ).
        FILTER ( ?c != ns3:Frame ).
        FILTER ( ?c != ns3:NamedEntity ).
        
        # -- Filter phenomena/relation
        FILTER NOT EXISTS {?amrC rdfs:subClassOf amr:AMR_Relation_Concept ;
                               amr:fromAmrLk ?c.}
        
        # -- Filter bugs
        FILTER NOT EXISTS { ?c rdfs:comment 'bug'. }
    """,
    'binding': """
        # -- Label
        BIND (strafter(str(?c), '#') AS ?cLabel1)
        BIND (strafter(str(?c), str(ns1:)) AS ?cLabel2)
        BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
        
        # -- New concept
        BIND (CONCAT(str(amr:), 'concept') AS ?n1).
        BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
        BIND (uri(?n2) AS ?newConcept).
    """
    }


rule_set['reclassify-concept-3'] = {
    'label': "reclassify-concept-3",
    'comment': ("Reclassify LK concept to AMR Predicat concept for concept: ",
                " (1) corresponding to a frame,",
                " (2) not corresponding to a relation."),
    'construction': """
        # -- New concept
        ?newConcept rdfs:subClassOf amr:AMR_Predicat_Concept ;
            amr:label ?cLabel ;
            amr:fromAmrLk ?c. 
            
        # -- Classification of the original concept as LinkedData (for tracing) 
        ?c rdfs:subClassOf amr:AMR_Linked_Data.
    """,
    'clause': """
        # -- Identify LK frame concepts
        ?c a ns3:Frame.
        
        # -- Filter phenomena/relation
        FILTER NOT EXISTS {?amrC rdfs:subClassOf amr:AMR_Relation_Concept ;
                               amr:fromAmrLk ?c.}
        
        # -- Filter bugs
        FILTER NOT EXISTS { ?c rdfs:comment 'bug'. }
    """,
    'binding': """
        # -- Label
        BIND (strafter(str(?c), '#') AS ?cLabel1)
        BIND (strafter(str(?c), str(ns1:)) AS ?cLabel2)
        BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
        
        # -- New concept
        BIND (CONCAT(str(amr:), 'concept') AS ?n1).
        BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
        BIND (uri(?n2) AS ?newConcept).
    """
    }


rule_set['reclassify-concept-4'] = {
    'label': "reclassify-concept-4",
    'comment': ("Reclassify LK concept to AMR Term concept for concept: ",
                " (1) corresponding to a term (ns2 concept),",
                " (2) not corresponding to a phenomena."),
    'construction': """
        # -- New concept
        ?newConcept rdfs:subClassOf amr:AMR_Term_Concept ;
            amr:label ?cLabel ;
            amr:fromAmrLk ?c.  
            
        # -- Classification of the original concept as LinkedData (for tracing) 
        ?c rdfs:subClassOf amr:AMR_Linked_Data.
    """,
    'clause': """
        # -- Identify LK term concepts
        ?c a ns3:Concept.
        FILTER ( CONTAINS(str(?c), str(ns2:)) ).
        FILTER ( ?c != ns3:Frame ).
        FILTER ( ?c != ns3:NamedEntity ).
        
        # -- Filter phenomena/relation
        FILTER NOT EXISTS {?amrC rdfs:subClassOf amr:AMR_Relation_Concept ;
                               amr:fromAmrLk ?c.}
        
        # -- Filter bugs
        FILTER NOT EXISTS { ?c rdfs:comment 'bug'. }
    """,
    'binding': """
        # -- Label
        BIND (strafter(str(?c), '#') AS ?cLabel1)
        BIND (strafter(str(?c), str(ns1:)) AS ?cLabel2)
        BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
        
        # -- New concept
        BIND (CONCAT(str(amr:), 'concept') AS ?n1).
        BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
        BIND (uri(?n2) AS ?newConcept).
    """
    }


rule_set['reclassify-concept-5'] = {
    'label': "reclassify-concept-5",
    'comment': ("Reclassify LK concept to AMR Term concept for concept: ",
                " (1) corresponding to a named entity,",
                " (2) not corresponding to a relation."),
    'construction': """
        # -- New concept
        ?newConcept rdfs:subClassOf amr:AMR_Term_Concept ;
            amr:label ?cLabel ;
            amr:fromAmrLk ?c.
            
        # -- Classification of the original concept as LinkedData (for tracing) 
        ?c rdfs:subClassOf amr:AMR_Linked_Data.
    """,
    'clause': """
        # -- Identify LK named entity concepts
        ?c a ns3:NamedEntity.
        
        # -- Filter phenomena/relation
        FILTER NOT EXISTS {?amrC rdfs:subClassOf amr:AMR_Relation_Concept ;
                               amr:fromAmrLk ?c.}
        
        # -- Filter bugs
        FILTER NOT EXISTS { ?c rdfs:comment 'bug'. }
    """,
    'binding': """
        # -- Label
        BIND (strafter(str(?c), '#') AS ?cLabel1)
        BIND (strafter(str(?c), str(ns1:)) AS ?cLabel2)
        BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
        
        # -- New concept
        BIND (CONCAT(str(amr:), 'concept') AS ?n1).
        BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
        BIND (uri(?n2) AS ?newConcept).
    """
    }


# ---------------------------------------------
# Reclassification of roles *** OLD ***
# ---------------------------------------------

# rule_set['reclassify-role-1'] = {
#     'label': "reclassify-role-1",
#     'comment': "Reclassify role to AMR core role",
#     'construction': """
#         # -- New AMR core role
#         ?newRole rdfs:subClassOf amr:AMR_Core_Role ;
#             amr:label ?roleId ;
#             amr:fromAmrLk ?r.
#     """,
#     'clause': """
#         # -- Identify core role
#         ?r a ns1:FrameRole.
#     """,
#     'binding':"""
#         # -- Label
#         BIND (strafter(str(?r), '#') AS ?rLabel1)
#         BIND (strafter(str(?r), str(ns1:)) AS ?rLabel2)
#         BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel)
        
#         # -- Id
#         BIND (strbefore(?rLabel, '.') AS ?rolesetId)
#         BIND (strafter(?rLabel, '.') AS ?roleId)
        
#         # -- New role
#         BIND (CONCAT(str(amr:), 'role') AS ?n1).
#         BIND (CONCAT(?n1, '_', ?roleId) AS ?n2).
#         BIND (uri(?n2) AS ?newRole).
#     """
#     }

# rule_set['reclassify-role-2'] = {
#     'label': "reclassify-rolet-2",
#     'comment': "Reclassify role to AMR non-core role",
#     'construction': """
#         # -- New AMR non core role
#         ?newRole rdfs:subClassOf amr:AMR_NonCore_Role ;
#             amr:label ?rLabel ;
#             amr:fromAmrLk ?r.
#     """,
#     'clause': """
#         # -- Identify non core role
#         ?r a ns3:Role.
#         FILTER ( ?r != ns1:FrameRole ).
        
#         # -- Filter core role
#         FILTER NOT EXISTS {?newRole rdfs:subClassOf* amr:AMR_Role ;
#                                amr:fromAmrLk ?r.}
#         """,
#     'binding':"""
#         # -- Label
#         BIND (strafter(str(?r), '#') AS ?rLabel1)
#         BIND (strafter(str(?r), str(ns1:)) AS ?rLabel2)
#         BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel)
        
#         # -- New role
#         BIND (CONCAT(str(amr:), 'role') AS ?n1).
#         BIND (CONCAT(?n1, '_', ?rLabel) AS ?n2).
#         BIND (uri(?n2) AS ?newRole).
#     """
#     }


# ---------------------------------------------
# Set labels for variables and roles
# ---------------------------------------------

# rule_set['set-amr-label-for-variables'] = {
#     'label': "set-amr-label-for-variables",
#     'comment': "Set AMR label for all variables",
#     'construction': """
#         ?v amr:label ?vLabel.
#     """,
#     'clause': """
#         ?v a amr:AMR_Variable.
#         """,
#     'binding':"""
#         BIND (strafter(str(?v), '#') AS ?vLabel)
#     """
#     }

# rule_set['set-amr-label-for-roles'] = {
#     'label': "set-amr-label-for-variables",
#     'comment': "Set AMR label for all variables",
#     'construction': """
#         ?r amr:label ?rLabel.
#     """,
#     'clause': """
#         ?r rdfs:subClassOf* amr:AMR_Role.
#         """,
#     'binding':"""
#         BIND (strafter(str(?r), 'role_') AS ?rLabel)
#     """
#     }


# ---------------------------------------------
# Reification of some roles as first-class concept
# ---------------------------------------------

rule_set['reify-roles-as-concept'] = {
    'label': "reify-roles-as-concept",
    'comment': ("reify some LK roles to AMR Relation concept for: ",
                " (1) AMR relation with reification,",
                " (2) non core LK role ",
                " (3) matching between role and reification relation."),
    'construction': """
        # -- New concept
        ?newConcept rdfs:subClassOf amr:AMR_Predicat_Concept ;
            amr:isReifiedConcept true ;
            amr:label ?reificationConcept ;
            amr:fromAmrLk ?r. 
            
        # -- Classification of the original concept as LinkedData (for tracing) 
        ?r rdfs:subClassOf amr:AMR_Linked_Data.
    """,
    'clause': """
        # -- Identify AMR relation with reification
        ?reifiableRelation rdfs:subClassOf amr:AMR_Relation ;
            amr:hasReification true ;
            amr:hasRelationName ?relationName ;
            amr:hasReificationConcept ?reificationConcept.
        # -- old --- amr:hasReificationDomain ?reificationDomain ;
        # -- old --- amr:hasReificationRange ?reificationRange.
        
        # -- Identify non core role
        ?r a ns3:Role.
        FILTER ( ?r != ns1:FrameRole ).
        
        # -- Identify role and reification relation matching
        BIND (strafter(str(?r), str(ns1:)) AS ?rLabel1).
        BIND (strafter(str(?r), '#') AS ?rLabel2). 
        FILTER (?rLabel1 = ?relationName || ?rLabel2 = ?relationName).
    """,
    'binding': """        
        # -- New concept
        BIND (CONCAT(str(amr:), 'concept') AS ?n1).
        BIND (CONCAT(?n1, '_', ?relationName) AS ?n2).
        BIND (uri(?n2) AS ?newConcept).
    """
    }


# ---------------------------------------------
# Reclassification of variables
# ---------------------------------------------

rule_set['reclassify-existing-variable'] = {
    'label': "reclassify-existing-variable",
    'comment': "Reclassify existing LK variable to AMR variable",
    'construction': """
        # -- New variable
        ?newVariable a amr:AMR_Variable ;
            amr:label ?vLabel ;
            amr:fromAmrLk ?v.
            
        # -- Name relation
        ?newVariable amr:name ?vName.
            
        # -- Classification of the original concept as LinkedData (for tracing) 
        ?v rdfs:subClassOf amr:AMR_Linked_Data.
    """,
    'clause': """
        # -- Identify variable
        ?c rdfs:subClassOf* amr:AMR_Concept ;
            amr:fromAmrLk ?linkedC.
        ?v a ?linkedC.
        
        # -- Identify rdfs:label (corresponding to name relation)
        OPTIONAL {?v rdfs:label ?vName}
    """,
    'binding': """
        # -- New label
        BIND (strafter(str(?v), '#') AS ?vLabel)
        
        # -- New variable
        BIND (CONCAT(str(amr:), 'variable') AS ?n1).
        BIND (CONCAT(?n1, '_', ?vLabel) AS ?n2).
        BIND (uri(?n2) AS ?newVariable).
    """
    }


rule_set['add-new-variable-for-reified-concept'] = {
    'label': "add-new-variable-for-reified-concept",
    'comment': "Add new variable for reified concept",
    'construction': """
        # -- New variable
        ?newVariable a amr:AMR_Variable ;
            amr:isReifiedVariable true ;
            amr:label ?vLabel.
            
        # -- Linking new variable to LK role (for tracing) 
        ?newVariable a ?r.
    """,
    'clause': """
        # -- Identify reified concept
        ?reifiedConcept rdfs:subClassOf amr:AMR_Predicat_Concept ;
            amr:isReifiedConcept true ;
            amr:label ?cLabel ;
            amr:fromAmrLk ?r. 
    		# -- old --- ?vIn ?r ?vOut.        
    """,
    'binding': """# -- New label
        BIND (strafter(str(?r), '#') AS ?rLabel1)
        BIND (strafter(str(?r), str(ns1:)) AS ?rLabel2)
        BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel)
        BIND (SUBSTR(?rLabel, 1, 1) AS ?vLetter)
        BIND (CONCAT(?vLetter, '9') AS ?vLabel).
        
        # -- old ---
        # -- old --- BIND (strafter(str(?vIn), '#') AS ?vl1).
        # -- old --- BIND (CONCAT(?vl1, ?vLetter) AS ?vl2).
        # -- old --- BIND (CONCAT(?vl2, strafter(str(?vOut), '#')) AS ?vl3).
        
        # -- New variable
        BIND (CONCAT(str(amr:), 'variable') AS ?n1).
        BIND (CONCAT(?n1, '_', ?vLabel) AS ?n2).
        BIND (uri(?n2) AS ?newVariable).
    """
    }


# ---------------------------------------------
# Addition of leafs
# ---------------------------------------------

rule_set['add-amr-leaf-for-reclassified-concept'] = {
    'label': "add-amr-leaf-for-reclassified-concept",
    'comment': "Add AMR leaf corresponding to concept/variable relation",
    'construction': """
        # -- New leaf
        ?newLeaf a amr:AMR_Leaf ;
            amr:hasVariable ?newV ;
            amr:hasConcept ?c.
    """,
    'clause': """
        # -- Identify concept
        ?c rdfs:subClassOf* amr:AMR_Concept ;
            amr:label ?cLabel ;
            amr:fromAmrLk ?linkedC.
        
        # -- Identify new variable
        ?newV a amr:AMR_Variable ;
            amr:label ?vLabel ;
            amr:fromAmrLk ?v.
            
        # -- identify matching between concept and (LK) variable
        ?v a ?linkedC.
        """,
    'binding':"""
        # -- New labels
        # -- old --- BIND (strafter(str(?linkedC), '#') AS ?cLabel1)
        # -- old --- BIND (strafter(str(?linkedC), str(ns1:)) AS ?cLabel2)
        # -- old --- BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
        # -- old --- BIND (strafter(str(?v), '#') AS ?vLabel)
        
        # -- New leaf
        BIND (CONCAT(str(amr:), 'leaf') AS ?n1).
        BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
        BIND (CONCAT(?n2, '_', ?vLabel) AS ?n3).
        BIND (uri(?n3) AS ?newLeaf).
    """
    }


rule_set['add-amr-leaf-for-reified-concept'] = {
    'label': "add-amr-leaf-for-reified-concept",
    'comment': "Add AMR leaf corresponding to reified concept (with new variable)",
    'construction': """
        # -- New leaf
        ?newLeaf a amr:AMR_Leaf ;
            amr:isReifiedLeaf true ;
            amr:hasVariable ?v ;
            amr:hasConcept ?c.
    """,
    'clause': """
        # -- Identify reified concept
        ?c rdfs:subClassOf* amr:AMR_Concept ;
            amr:isReifiedConcept true ;
            amr:label ?cLabel ;
            amr:fromAmrLk ?linkedC.
        
        # -- Identify reified variable
        ?v a amr:AMR_Variable ;
            amr:isReifiedVariable true ;
            amr:label ?vLabel.
            
        # -- Identify matching between reified concept and variable
        ?v a ?linkedC.
        """,
    'binding':"""
        # -- New labels
        # -- old --- BIND (strafter(str(?linkedC), '#') AS ?cLabel1)
        # -- old --- BIND (strafter(str(?linkedC), str(ns1:)) AS ?cLabel2)
        # -- old --- BIND (IF (STRLEN(?cLabel1) > 0, ?cLabel1, ?cLabel2) AS ?cLabel)
        # -- old --- BIND (strafter(str(?v), '#') AS ?vLabel)
        
        # -- New leaf
        BIND (CONCAT(str(amr:), 'leaf') AS ?n1).
        BIND (CONCAT(?n1, '_', ?cLabel) AS ?n2).
        BIND (CONCAT(?n2, '_', ?vLabel) AS ?n3).
        BIND (uri(?n3) AS ?newLeaf).
    """
    }


# ---------------------------------------------
# Addition of edges
# ---------------------------------------------

rule_set['add-amr-edge-for-core-relation'] = {
    'label': "add-amr-edge-for-core-relation",
    'comment': ("Add AMR edge corresponding to: ",
                " (x) LK core relation between two leafs"),
    'construction': """
        # -- New edge
        ?newEdge a amr:AMR_Edge ;
            amr:hasRoleID ?roleId.
            # -- old --- amr:hasRolesetID ?rolesetId ;
            # -- old --- amr:hasRoleID ?roleId ;
            # -- old --- amr:hasRoleTag ?roleTag.
            
        # -- New relation between leafs
		?leaf1 ?newEdge ?leaf2.
    """,
    'clause': """
        # -- Identify LK relation
        ?r a ?amrLkRelation.
            VALUES ?amrLkRelation {ns1:FrameRole ns3:Role}
            
        # -- Filter reified role
		FILTER NOT EXISTS { 
            ?newConcept rdfs:subClassOf amr:AMR_Predicat_Concept ;
                amr:isReifiedConcept true ;
                amr:fromAmrLk ?r.
                }  
            
        # -- identify leafs and variables
        ?leaf1 amr:hasVariable ?v1.
        ?leaf2 amr:hasVariable ?v2.
        ?v1 a amr:AMR_Variable ;
            amr:label ?v1Label ;
            amr:fromAmrLk ?v1Link.
        ?v2 a amr:AMR_Variable ;
            amr:label ?v2Label ;
            amr:fromAmrLk ?v2Link.
            
        # -- Identify matching
		?v1Link ?r ?v2Link.
        """,
    'binding':"""
        # -- New labels
        BIND (strafter(str(?r), '#') AS ?rLabel1)
        BIND (strafter(str(?r), str(ns1:)) AS ?rLabel2)
        BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel)
        BIND (IF (CONTAINS(?rLabel, "."), 
                  strafter(?rLabel, '.'), 
                  ?rLabel) AS ?roleId)
        # -- old --- BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel3)
        # -- old --- BIND (IF (CONTAINS(?rLabel3, "."), ?rLabel3, CONCAT("amr.", ?rLabel3)) AS ?roleTag)
        # -- old --- BIND (strbefore(?roleTag, '.') AS ?rolesetId)
        # -- old --- BIND (strafter(?roleTag, '.') AS ?roleId)
        # -- old --- BIND (IF (STRLEN(?roleId) > 0, ?roleId, ?rLabel3) AS ?rLabel)
        
        # -- New edge
        BIND (CONCAT(str(amr:), 'edge') AS ?n1).
        BIND (CONCAT(?n1, '_', ?v1Label) AS ?n2).
        BIND (CONCAT(?n2, '_', ?roleId) AS ?n3).
        BIND (CONCAT(?n3, '_', ?v2Label) AS ?n4).
        BIND (uri(?n4) AS ?newEdge).
    """
    }

rule_set['add-amr-edge-for-reified-concept'] = {
    'label': "add-amr-edge-for-reified-concept",
    'comment': ("Add AMR edge corresponding to: ",
                " (x) reification of LK non-core relation to concept"),
    'construction': """
        # -- New edge (ARG0)
        ?newEdge0 a amr:AMR_Edge ;
            amr:hasRoleID ?roleId0.
            # -- old --- amr:hasRolesetID ?rolesetId0 ;
            # -- old --- amr:hasRoleID ?roleId0 ;
            # -- old --- amr:hasRoleTag ?roleTag0.
           
        # -- New edge (ARG1)
        ?newEdge1 a amr:AMR_Edge ;
            amr:hasRoleID ?roleId1.
            # -- old --- amr:hasRolesetID ?rolesetId1 ;
            # -- old --- amr:hasRoleID ?roleId1 ;
            # -- old --- amr:hasRoleTag ?roleTag1.
            
        # -- New relations between leafs
		?reifiedLeaf ?newEdge0 ?leaf1.
		?reifiedLeaf ?newEdge1 ?leaf2.
    """,
    'clause': """
        # -- Identify reified leaf
        ?reifiedLeaf a amr:AMR_Leaf ;
            amr:isReifiedLeaf true ;
            amr:hasVariable ?reifiedVariable ;
            amr:hasConcept ?reifiedConcept.
        ?reifiedVariable amr:label ?rvLabel.
            
        # -- Identify LK relation corresponding to the reified concept
        ?reifiedConcept amr:fromAmrLk ?r. 
            
        # -- Identify leafs and variables
        ?leaf1 amr:hasVariable ?v1.
        ?leaf2 amr:hasVariable ?v2.
        ?v1 a amr:AMR_Variable ;
            amr:label ?v1Label ;
            amr:fromAmrLk ?v1Link.
        ?v2 a amr:AMR_Variable ;
            amr:label ?v2Label ;
            amr:fromAmrLk ?v2Link.
            
        # -- Identify matching
		?v1Link ?r ?v2Link.
        """,
    'binding':"""
        # -- New labels
        # -- old --- BIND (strafter(str(?r), '#') AS ?rLabel1)
        # -- old --- BIND (strafter(str(?r), str(ns1:)) AS ?rLabel2)
        # -- old --- BIND (IF (STRLEN(?rLabel1) > 0, ?rLabel1, ?rLabel2) AS ?rLabel3)
        # -- old --- BIND (IF (CONTAINS(?rLabel3, "."), 
        # -- old ---           ?rLabel3, 
        # -- old ---           CONCAT("amr.", ?rLabel3)) AS ?roleTag)
        # -- old --- BIND (strbefore(?roleTag, '.') AS ?rolesetId)
        # -- old --- BIND (strafter(?roleTag, '.') AS ?roleId)
        # -- old --- BIND (IF (STRLEN(?roleId) > 0, ?roleId, ?rLabel3) AS ?rLabel)
        
        # -- New labels for ARG0 edge
        # -- old --- BIND ("amr.ARG0" AS ?roleTag0)
        # -- old --- BIND (strbefore(?roleTag0, '.') AS ?rolesetId0)
        # -- old --- BIND (strafter(?roleTag0, '.') AS ?roleId0)
        
        # -- New labels for ARG1 edge
        # -- old --- BIND ("amr.ARG1" AS ?roleTag1)
        # -- old --- BIND (strbefore(?roleTag1, '.') AS ?rolesetId1)
        # -- old --- BIND (strafter(?roleTag1, '.') AS ?roleId1)
        
        # -- New labels
        BIND ("ARG0" AS ?roleId0)
        BIND ("ARG1" AS ?roleId1)
        
        # -- New edge (ARG0)
        BIND (CONCAT(str(amr:), 'edge') AS ?n01).
        BIND (CONCAT(?n01, '_', ?rvLabel) AS ?n02).
        BIND (CONCAT(?n02, '_', ?roleId0) AS ?n03).
        BIND (CONCAT(?n03, '_', ?v1Label) AS ?n04).
        BIND (uri(?n04) AS ?newEdge0).
        
        # -- New edge (ARG1)
        BIND (CONCAT(str(amr:), 'edge') AS ?n11).
        BIND (CONCAT(?n11, '_', ?rvLabel) AS ?n12).
        BIND (CONCAT(?n12, '_', ?roleId1) AS ?n13).
        BIND (CONCAT(?n13, '_', ?v2Label) AS ?n14).
        BIND (uri(?n14) AS ?newEdge1).
    """
    }

rule_set['add-amr-edge-for-name-relation'] = {
    'label': "add-amr-edge-for-name-relation",
    'comment': ("Add AMR edge corresponding to: ",
                " (x) leaf with name relation."),
    'construction': """
        # -- New value
        ?newValue a amr:AMR_Value ;
            rdfs:label ?vName.
        
        # -- New edge
        ?newEdge a amr:AMR_Edge ;
            amr:hasRoleID ?roleId.
            # -- old --- amr:hasRolesetID ?rolesetId ;
            # -- old --- amr:hasRoleID ?roleId ;
            # -- old --- amr:hasRoleTag ?roleTag.
            
        # -- New relation between leaf and value
		?leaf ?newEdge ?newValue.
    """,
    'clause': """
        # -- Identify leaf with name relation
        ?leaf a amr:AMR_Leaf ;
            amr:hasVariable ?v ;
            amr:hasConcept ?c.
        ?v a amr:AMR_Variable ;
            amr:label ?vLabel ;
            amr:name ?vName.
	""",
    'binding':"""
        # -- New value
        BIND (REPLACE(?vName, ' ', "") AS ?nvLabel).
        BIND (CONCAT(str(amr:), 'value') AS ?nv1).
        BIND (CONCAT(?nv1, '_', ?nvLabel) AS ?nv2).
        BIND (uri(?nv2) AS ?newValue).
        
        # -- New labels
        BIND ('amr.name' AS ?rLabel).
        BIND (strbefore(?rLabel, '.') AS ?rolesetId).
        BIND (strafter(?rLabel, '.') AS ?roleId).
        BIND (?rLabel AS ?roleTag).
        # -- old --- BIND (strafter(str(?v), '#') AS ?vLabel).
        
        # -- New edge
        BIND (CONCAT(str(amr:), 'edge') AS ?ne1).
        BIND (CONCAT(?ne1, '_', ?vLabel) AS ?ne2).
        BIND (CONCAT(?ne2, '_', ?roleId) AS ?ne3).
        BIND (CONCAT(?ne3, '_', ?nvLabel) AS ?ne4).
        BIND (uri(?ne4) AS ?newEdge).
    """
    }


rule_set['add-amr-edge-for-quant-relation'] = {
    'label': "add-value-for-quant-relation",
    'comment': ("Add AMR edge corresponding to: ",
                " (x) leaf with quant relation."),
    'construction': """
        # -- New value
        ?newValue a amr:AMR_Value ;
            rdfs:label ?vLabel.
        
        # -- New edge
        ?newEdge a amr:AMR_Edge ;
            amr:hasRoleID ?roleId.
            # -- old --- amr:hasRolesetID ?rolesetId ;
            # -- old --- amr:hasRoleID ?roleId ;
            # -- old --- amr:hasRoleTag ?roleTag.
            
        # -- New relation between leaf and value
		?leaf ?newEdge ?newValue.
    """,
    'clause': """
        # -- Identify leaf
        ?leaf a amr:AMR_Leaf ;
            amr:hasVariable ?v ;
            amr:hasConcept ?c.
        ?v a amr:AMR_Variable ;
            amr:label ?vLabel ;
            amr:fromAmrLk ?vLink.
            
        # -- Identify matching with quant relation (ns2:quant)
        ?vLink ns2:quant ?vQuant.
	""",
    'binding':"""
        # -- New label
        BIND (REPLACE(?vQuant, ' ', "") AS ?nvLabel).
        
        # -- New value
        BIND (CONCAT(str(amr:), 'value') AS ?nv1).
        BIND (CONCAT(?nv1, '_', ?nvLabel) AS ?nv2).
        BIND (uri(?nv2) AS ?newValue).
        
        # -- New labels
        BIND ('amr.quant' AS ?rLabel).
        BIND (strbefore(?rLabel, '.') AS ?rolesetId).
        BIND (strafter(?rLabel, '.') AS ?roleId).
        BIND (?rLabel AS ?roleTag).
        # -- old --- BIND (strafter(str(?v), '#') AS ?vLabel).
        
        # -- New edge
        BIND (CONCAT(str(amr:), 'edge') AS ?ne1).
        BIND (CONCAT(?ne1, '_', ?vLabel) AS ?ne2).
        BIND (CONCAT(?ne2, '_', ?roleId) AS ?ne3).
        BIND (CONCAT(?ne3, '_', ?nvLabel) AS ?ne4).
        BIND (uri(?ne4) AS ?newEdge).
    """
    }


rule_set['add-amr-edge-for-polarity-relation'] = {
    'label': "add-amr-edge-for-polarity-relation",
    'comment': ("Add AMR edge corresponding to: ",
                " (x) leaf with polarity relation."),
    'construction': """
        # -- New value
        ?newValue a amr:AMR_Value ;
            rdfs:label ?nvLabel.
        
        # -- New edge
        ?newEdge a amr:AMR_Edge ;
            amr:hasRoleID ?roleId.
            # -- old --- amr:hasRolesetID ?rolesetId ;
            # -- old --- amr:hasRoleID ?roleId ;
            # -- old --- amr:hasRoleTag ?roleTag.
            
        # -- New relation between leaf and value
		?leaf ?newEdge ?newValue.
    """,
    'clause': """
        # -- Identify leaf
        ?leaf a amr:AMR_Leaf ;
            amr:hasVariable ?v ;
            amr:hasConcept ?c.
        ?v a amr:AMR_Variable ;
            amr:label ?vLabel ;
            amr:fromAmrLk ?vLink.
            
        # -- Identify matching with polarity relation (ns2:polarity)
        ?vLink ns2:polarity ?vPolarity.
	""",
    'binding':"""
        # -- New label
        BIND (REPLACE(?vPolarity, ' ', "") AS ?nvl1).
        BIND (REPLACE(?nvl1, '-', 'negative') AS ?nvLabel).
        
        # -- New value
        BIND (CONCAT(str(amr:), 'value') AS ?nv1).
        BIND (CONCAT(?nv1, '_', ?nvLabel) AS ?nv2).
        BIND (uri(?nv2) AS ?newValue).
        
        # -- New labels
        BIND ('amr.polarity' AS ?rLabel).
        BIND (strbefore(?rLabel, '.') AS ?rolesetId).
        BIND (strafter(?rLabel, '.') AS ?roleId).
        BIND (?rLabel AS ?roleTag).
        # -- old --- BIND (strafter(str(?v), '#') AS ?vLabel).
        
        # -- New edge
        BIND (CONCAT(str(amr:), 'edge') AS ?ne1).
        BIND (CONCAT(?ne1, '_', ?vLabel) AS ?ne2).
        BIND (CONCAT(?ne2, '_', ?roleId) AS ?ne3).
        BIND (CONCAT(?ne3, '_', ?nvLabel) AS ?ne4).
        BIND (uri(?ne4) AS ?newEdge).
    """
    }


# ---------------------------------------------
# Update leafs and edges *** TODO ***
# ---------------------------------------------

rule_set['update-amr-edge-role-1'] = {
    'label': "update-amr-edge-role-1",
    'comment': ("Update AMR edge by adding relation with AMR_Role",
                "(using comparison between label and role id)"),
    'construction': """
        ?edge amr:hasAmrRole ?role.
    """,
    'clause': """
        ?edge a amr:AMR_Edge ; 
            amr:hasRoleID ?edgeRoleId.
        ?role rdfs:subClassOf* amr:AMR_Role ;
            amr:label ?roleLabel.
        FILTER ( ?roleLabel != '' ).
        FILTER ( ?edgeRoleId = ?roleLabel ).
    """,
    'binding': ''
    }

# rule_set['update-amr-edge-role-2'] = {
#     'label': "update-amr-edge-role-2",
#     'comment': ("Update AMR edge by adding relation with AMR_Role",
#                 "(using comparison between label and role tag)"),
#     'construction': """
#         ?edge amr:hasAmrRole ?role.
#     """,
#     'clause': """
#         ?edge a amr:AMR_Edge ; 
#             amr:hasRoleTag ?edgeRoleId.
#         ?role rdfs:subClassOf* amr:AMR_Role ;
#             amr:label ?roleLabel.
#         FILTER ( ?roleLabel != '' ).
#         FILTER ( ?edgeRoleId = ?roleLabel ).
#     """,
#     'binding': ''
#     }


# ---------------------------------------------
# Preprocessing: Addition of root
# ---------------------------------------------

rule_set['add-amr-root'] = {
    'label': "add-amr-root",
    'comment': ("Add root instance in AMR_Root"),
    'construction': """
        # -- New root
        ?newRoot a amr:AMR_Root ;
            amr:hasRootLeaf ?leaf ;
            amr:hasSentenceID ?graphId ;
            amr:hasSentenceStatement ?graphSentence ;
            amr:fromAmrLk ?r.
    """,
    'clause': """
        # -- Identify root in AMR-LK
        ?r a ns3:AMR.
        ?r ns3:root ?lkRoot.
        ?r ns3:has-id ?graphId.
        ?r ns3:has-sentence ?graphSentence.
        ?leaf amr:hasVariable ?varRoot.
        ?varRoot a amr:AMR_Variable ;
            amr:fromAmrLk ?lkRoot.
		""",
    'binding':"""
        # -- New root
        BIND (CONCAT(str(amr:), 'root') AS ?n1).
        BIND (CONCAT(?n1, '_', ?graphId) AS ?n2).
        BIND (uri(?n2) AS ?newRoot).
    """
    }

