#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: AMR CTR at 'Net Expansion' level
#------------------------------------------------------------------------------
# Module grouping compositional transduction rule_sets (CTR) for the analysis of 
# AMR structures, at 'Net Expansion' level
#==============================================================================

#==============================================================================
# Query Builder Imports
#==============================================================================

#import query_builder.construct as construct
from query_builder.builders import *


#==============================================================================
# CTR Set (CTR as Sparql querries)
#==============================================================================

rule_set = {}

 
# ---------------------------------------------
# Atomic Extraction: Class Net
# ---------------------------------------------

rule_set['create-atom-class-net'] = {
    'label': "create-atom-class-net",
    'comment': "Create Atom Class Net from AMR Term Concept",
    'construction': f"""           
        {atom_class_net.construct(base_node='?leaf1', 
                                  class_name='?conceptName',
                                  naming='?conceptName')}
        
        {atom_class_net.propagate_relations()}
    """,
    'clause': f"""
        # -- Identify Class covering a single leaf
        ?leaf1 a amr:AMR_Leaf ;
            amr:hasConcept ?leaf1Concept ;
            amr:hasVariable ?variable.
        ?variable amr:label ?varLabel.
        ?leaf1Concept rdfs:subClassOf amr:AMR_Term_Concept.
        ?leaf1Concept amr:label ?conceptName.
            
        {atom_class_net.complete_clauses_for_construction('?leaf1')}
    """,
    'binding': f"""
        {atom_class_net.bind_uri('?conceptName', '?varLabel')}
    """
}

 
# ---------------------------------------------
# Atomic Extraction: Individual Net
# ---------------------------------------------

rule_set['create-individual-net-1'] = {
    'label': "create-individual-net-1",
    'comment': ("Create Individual Net from AMR Leaf: ",
                " (1) with variable corresponding to a named entity ",
                " (2) without link to domain edge"),
    'construction': f"""
        {individual_net.construct(base_node='?baseLeaf',
                                  naming='?conceptName', 
                                  mother_class_net='?classNet',
                                  individual_label='?valueLabel')}
        
        {individual_net.propagate_relations()}
    """,
    'clause': f"""
        # -- Identify name relation between a leaf and a value
        ?baseLeaf a amr:AMR_Leaf ;
            amr:hasConcept ?concept ;
            amr:hasVariable ?variable.
		?baseLeaf ?edge ?value.
        ?edge amr:hasRoleID ?roleID.
            VALUES ?roleID {{'name'}}
        ?value a amr:AMR_Value ;
            rdfs:label ?valueLabel.
        ?classNet a [rdfs:subClassOf* net:Class_Net] ;
        		net:coverBaseNode ?baseLeaf ;
            net:hasClassName ?conceptName. 
            
        {individual_net.complete_clauses_for_construction('?baseLeaf')}
    """,
    'binding': f"""
        {individual_net.bind_uri('?conceptName', '?valueLabel')}
    """
}

 
# ---------------------------------------------
# Atomic Extraction: Property Net
# ---------------------------------------------

rule_set['create-atom-property-net-1'] = {
    'label': "create-atom-property-net-1",
    'comment': ("Create Atom Property Net: ",
                " (1) from AMR Predicat Concept"),
    'construction': f"""
        {atom_property_net.construct(base_node='?baseLeaf', 
                                     naming='?propertyName',
                                     core_role='true',
                                     target_argument_node='?argLeaf',
                                     property_type='owl:ObjectProperty',
                                     property_name='?propertyName',
                                     property_name01='?propertyName01',
                                     property_name10='?propertyName10',
                                     property_name12='?propertyName12')}
           
        {atom_property_net.propagate_relations()}
    """,
    'clause': f"""
        # -- Identify property covering single leaf
        ?baseLeaf a amr:AMR_Leaf ;
            amr:hasConcept ?concept ;
            amr:hasVariable ?variable.
        ?variable amr:label ?varLabel.
        ?concept rdfs:subClassOf amr:AMR_Predicat_Concept ;
            amr:label ?predicateLabel.  
        OPTIONAL {{
                    ?baseLeaf ?relation ?argLeaf.    
                    ?relation amr:hasAmrRole ?someArgRole.
                  }}
            
        {atom_property_net.complete_clauses_for_construction('?baseLeaf')}
    """,
    'binding': f"""
        BIND (STRBEFORE(?predicateLabel, '-') AS ?pName1).
		BIND (IF (STRLEN(?pName1) > 0, 
            ?pName1, ?predicateLabel) AS ?propertyName)
		BIND (IF (STRLEN(?pName1) > 0, 
            CONCAT(?propertyName, 'ing'), ?predicateLabel) AS ?propertyName01)
		BIND (IF (STRLEN(?pName1) > 0, 
            CONCAT(?propertyName, '-by'), ?predicateLabel) AS ?propertyName10)
		BIND (IF (STRLEN(?pName1) > 0, 
            CONCAT(?propertyName, '-of'), ?predicateLabel) AS ?propertyName12)
            
        {atom_property_net.bind_uri('?propertyName', '?varLabel')}
    """
}

# -- old ---
# rule_set['create-atom-property-net-2'] = {
#     'label': "create-atom-property-net-2",
#     'comment': ("Create Property Net from AMR Predicat Concept: ",
#                 " with variable corresponding to a negative polarity value "),
#     'construction': f"""
#         # -- New Property Net
#         ?newNet a net:Atom_Property_Net ;
#             net:isCoreRoleLinked true ;
#             net:hasStructure ?sentenceRef ;
#             net:coverBaseNode ?baseLeaf ;
#             net:targetArgumentNode ?argLeaf ;
#             net:hasPropertyType owl:ObjectProperty ;
#             net:hasPropertyName ?propertyName ;
#             net:hasPropertyName01 ?propertyName01 ;
#             net:hasPropertyName10 ?propertyName10 ;
#             net:hasPropertyName12 ?propertyName12.
            
#         # -- Propagation of relations (from nodes to nets)
#         ?inRelationRole a net:Relation.
#         ?inNet ?inRelationRole ?newNet.
#         ?outRelationRole a net:Relation.
#         ?newNet ?outRelationRole ?outNet.
#     """,
#     'clause': """
#         # -- Identify property covering single leaf
#         ?baseLeaf a amr:AMR_Leaf ;
#             amr:hasConcept ?concept ;
#             amr:hasVariable ?variable.
#         ?variable amr:label ?varLabel ;
#             amr:polarity ?polarityValue.
#         ?polarityValue rdfs:label 'negative'.
#         ?concept rdfs:subClassOf amr:AMR_Predicat_Concept ;
#             amr:label ?predicateLabel.  
#         OPTIONAL {?baseLeaf ?relation ?argLeaf.    
#                   ?relation amr:hasAmrRole ?someArgRole.}
        
#         # -- Identify structure
#         ?root a amr:AMR_Root ;
#             amr:hasSentenceID ?sentenceRef.
            
#         # -- Identify inbound relations linked to the base leaf (for propagation) 
#         	OPTIONAL {?inNet a [rdfs:subClassOf* net:Net] ;
#                        net:coverBaseNode ?inLeaf.
#                   ?inLeaf ?inRelationEdge ?baseLeaf.    
#                   ?inRelationEdge amr:hasAmrRole ?inRelationRole.}
            
#         # -- Identify outgoing relations linked to the base leaf (for propagation) 
#         	OPTIONAL {?outNet a [rdfs:subClassOf* net:Net] ;
#                        net:coverBaseNode ?outLeaf.
#                   ?baseLeaf ?outRelationEdge ?outLeaf.    
#                   ?outRelationEdge amr:hasAmrRole ?outRelationRole.}
#     """,
#     'binding': f"""
#         # -- New Property Name
#     		BIND (STRBEFORE(?predicateLabel, '-') AS ?basePropertyName).
#         	BIND (CONCAT('not-', ?basePropertyName) AS ?propertyName).
#     		BIND (CONCAT(?propertyName, 'ing') AS ?propertyName01).
#     		BIND (CONCAT(?propertyName, '-by') AS ?propertyName10).
#     		BIND (CONCAT(?propertyName, '-of') AS ?propertyName12).
            
#         # -- New Property Net
# 		BIND (CONCAT(str(net:), 'atomProperty') AS ?n1).
# 		BIND (CONCAT(?n1, '_', ?propertyName) AS ?n2).
# 		BIND (CONCAT(?n2, '_', ?varLabel) AS ?n3).
#         BIND (uri(?n3) AS ?newNet).
#     """
# }
# -- ---

# -- old ---
# rule_set['create-atom-property-net-3'] = {
#     'label': "create-atom-property-net-3",
#     'comment': ("Create Atom Property Net: ",
#                 " (1) from AMR Non Core Role Edge"),
#     'construction': f"""
#         # -- New Property Net
#         ?newNet a net:Atom_Property_Net ;
#             net:isCoreRoleLinked false ;
#             net:hasStructure ?sentenceRef ;
#             net:coverBaseNode ?baseLeaf ;
#             net:targetArgumentNode ?targetPropertyLeaf ;
#             net:hasPropertyType ?roleType ;
#             net:hasPropertyRole ?relationRole ;
#             net:hasPropertyName ?roleName.
            
#         # -- Propagation of relations (from nodes to nets)
#         ?inRelationRole a net:Relation.
#         ?inNet ?inRelationRole ?newNet.
#         ?outRelationRole a net:Relation.
#         ?newNet ?outRelationRole ?outNet.
#     """,
#     'clause': """
#         # -- Identify property covering relation role
#         ?relationRole rdfs:subClassOf amr:AMR_NonCore_Role ;
#             amr:getPropertyType ?roleType ;
#             amr:getDirectPropertyName ?roleName.
#         FILTER (?roleType = owl:ObjectProperty || 
#                 ?roleType = owl:DataProperty).
#         ?roleEdge amr:hasAmrRole ?relationRole.
#         ?baseLeaf ?roleEdge ?targetPropertyLeaf.
#         ?baseLeaf a amr:AMR_Leaf ;
#             amr:hasVariable ?variable.
#         ?variable amr:label ?varLabel.
        
#         # -- Identify structure
#         ?root a amr:AMR_Root ;
#             amr:hasSentenceID ?sentenceRef.
            
#         # -- Identify inbound relations linked to the base leaf (for propagation) 
#         	OPTIONAL {?inNet a [rdfs:subClassOf* net:Net] ;
#                        net:coverBaseNode ?inLeaf.
#                   ?inLeaf ?inRelationEdge ?baseLeaf.    
#                   ?inRelationEdge amr:hasAmrRole ?inRelationRole.}
            
#         # -- Identify outgoing relations linked to the base leaf (for propagation) 
#         	OPTIONAL {?outNet a [rdfs:subClassOf* net:Net] ;
#                        net:coverBaseNode ?outLeaf.
#                   ?baseLeaf ?outRelationEdge ?outLeaf.    
#                   ?outRelationEdge amr:hasAmrRole ?outRelationRole.}
#     """,
#     'binding': f"""
#         # -- New Property Net
# 		BIND (CONCAT(str(net:), 'atomProperty') AS ?n1).
# 		BIND (CONCAT(?n1, '_', str(?roleName)) AS ?n2).
# 		BIND (CONCAT(?n2, '_', str(?varLabel)) AS ?n3).
#         BIND (uri(?n3) AS ?newNet).
#     """
# }
# -- ---


# ---------------------------------------------
# Atomic Extraction: Value Net
# ---------------------------------------------

rule_set['create-value-net'] = {
    'label': "create-value-net",
    'comment': "Create Value Net from AMR Value",
    'construction': f"""           
        {value_net.construct(naming='?valueLabel',
                             value_label='?valueLabel')}

        {value_net.propagate_relations()}
    """,
    'clause': f"""
        # -- Identify Class covering a single leaf
        ?value a amr:AMR_Value ;
            rdfs:label ?valueLabel.
            
        {value_net.complete_clauses_for_construction('?value')}
    """,
    'binding': f"""
        {value_net.bind_uri('?valueLabel')}
    """
} 

# ---------------------------------------------
# Atomic Extraction: Phenomena Net
# ---------------------------------------------

rule_set['create-phenomena-net-1'] = {
    'label': "create-phenomena-net-1",
    'comment': ("Create Phenomena Net from AMR Relation Concept"),
    'construction': f"""
        {phenomena_net.construct(base_node='?baseLeaf', 
                                 naming='?phenomenaLabel',
                                 phenomena_type='?relationConceptPhenomena',
                                 phenomena_ref='?relationConceptLabel')}
       
       {phenomena_net.propagate_relations()}
    """,
    'clause': f"""
        # Identify relation concept (phenomena) covering a single leaf
        ?baseLeaf a amr:AMR_Leaf ;
            amr:hasConcept ?concept ;
            amr:hasVariable ?variable.
        ?variable amr:label ?varLabel.
        ?concept rdfs:subClassOf amr:AMR_Relation_Concept ;
            amr:hasPhenomenaLink ?relationConceptPhenomena ;  
            amr:label ?relationConceptLabel.  
        ?relationConceptPhenomena amr:label ?phenomenaLabel.
        ?baseLeaf ?relation ?argLeaf.    
        ?relation amr:hasAmrRole ?someArgRole.
            
        {phenomena_net.complete_clauses_for_construction('?baseLeaf')}
    """,
    'binding': f"""
        {phenomena_net.bind_uri('?phenomenaLabel', '?varLabel')}
    """
}


#==============================================================================
# Development Test
#==============================================================================
    
if __name__ == '__main__':
    
    print('\n' + ' *** Development Test ***')    
        
    print('\n' + ' -- test: Atom Class Net')
    print(atom_class_net)
        
    print('\n' + ' -- test: update a test query (1)')
    #rule = rule_set['create-atom-property-net-1']
    #rule = rule_set['create-value-net']
    rule = rule_set['create-individual-net-1']
    
    test_query_1 = f"""[...]
        CONSTRUCT {{
            {rule['construction']}
            
        
        }}
        WHERE {{
            {rule['clause']}
            
            {rule['binding']}
            
        }}
    """
    print(test_query_1)    
    
    print('\n' + ' *** - ***')
