#!/usr/bin/python3.10
# -*-coding:Utf-8 -*

#==============================================================================
# TENET: Extraction Rule Test
#------------------------------------------------------------------------------
# Script to test rules under development
#==============================================================================

import subprocess, os
import rdflib
from rdflib import Graph, Namespace, URIRef
from rdflib.namespace import NamespaceManager
from rdflib.term import _is_valid_uri

FILE_PATH = f'{os.path.dirname(os.path.abspath(__file__))}'
INPUT_DIR_PATH = f'{FILE_PATH}/input/'
OUTPUT_DIR_PATH = f'{FILE_PATH}/output/'
TEST_GRAPH = f'{INPUT_DIR_PATH}testGraph1.ttl'

from context import tenet
import utility

from tenet.febTransduction import rdfterm_computer



#==============================================================================
# Useful Methods
#==============================================================================

def load_test_graph():
    print(f'\n -- Test Graph Loading')
    graph = Graph()
    graph.bind('amr', Namespace('https://amr.tetras-libre.fr/rdf/schema#'))
    graph.bind('net', Namespace('https://tenet.tetras-libre.fr/semantic-net#'))
    graph.bind('base-out', Namespace('https://tenet.tetras-libre.fr/base-ontology#'))
    graph.parse(TEST_GRAPH)
    print(f" ----- Graph Loaded ({len(graph)})")
    return graph


#==============================================================================
# Development Test
#==============================================================================

def devtest_check_uri(uri):
    print(f'\n -- Test URI checking')
    print(f' ----- uri: {uri}')
    print(f' ----- check 1 (valid uri): {_is_valid_uri(uri)}')
    print(f' ----- check 2 (URIRef instance): {isinstance(uri, rdflib.term.URIRef)}')
    print(f' ----- check 3 (str instance): {isinstance(uri, str)}')
    print(f' ----- check 4 (not http uri): {uri.startswith("http")}')
    print(f' ----- check 5 (not single quote uri): {uri.startswith("<") & uri.endswith(">")}')
    
    
def devtest_create_uriref(graph, uri):
    print(f'\n -- Creation of URI Reference')
    assert isinstance(uri, str), f'URI is not a string'
    
    origin_uri = uri
    new_uri = None
    
    if (uri.startswith("<") & uri.endswith(">")):
        uri = uri[1:-1]
    
    if not isinstance(uri, rdflib.term.URIRef):
        uri = rdflib.URIRef(uri)
    assert isinstance(uri, rdflib.term.URIRef), f'Houston, we have a problem (1)'
    
    if uri.startswith("http"):
        new_uri = rdflib.URIRef(uri.n3(graph.namespace_manager))
        assert isinstance(new_uri, rdflib.term.URIRef), f'Houston, we have a problem (2)'
    else:
        new_uri = uri
        assert isinstance(new_uri, rdflib.term.URIRef), f'Houston, we have a problem (3)'

    assert _is_valid_uri(uri), f'Houston, we have a problem: URI is not a valid uri'
    print(f' ----- origin uri: {origin_uri}')
    print(f' ----- new uri: {new_uri}')
    print(f' ----- new uri checking as URIRef instance: {isinstance(new_uri, rdflib.term.URIRef)}')



#==============================================================================
# Unit Test
#==============================================================================

def unittest_produce_uriref(graph, uri):
    print(f'\n -- Produce valid URI Reference')
    try:
        uriref = rdfterm_computer.produce_uriref(graph, uri)
        if uriref is None:
            print(f' ----- None')
        else: # uriref is not None
            print(f' ----- origin uri: {uri}')
            print(f' ----- result uri: {uriref}')
            c_1 = _is_valid_uri(uriref)
            c_2 = isinstance(uriref, rdflib.term.URIRef)
            c_3 = isinstance(uriref, str)
            c_4 = not(uriref.startswith("http"))
            c_5 = not(uriref.startswith("<") & uriref.endswith(">"))
            assert c_1 & c_2 & c_3 & c_4 & c_5
            print(f' ----- result uri check: ok')
    except:
        print(f' ----- check 1 (valid uri): {c_1}')
        print(f' ----- check 2 (URIRef instance): {c_2}')
        print(f' ----- check 3 (str instance): {c_3}')
        print(f' ----- check 4 (not http uri): {c_4}')
        print(f' ----- check 5 (not single quote uri): {c_5}')
    

def unittest_failure_produce_uriref(graph, uri):
    print(f'\n -- * Failure to produce URI Reference *')
    try:
        uriref = rdfterm_computer.produce_uriref(graph, uri)
        print(f'Houston, we have a problem: production is successful')
    except:
        print(f' ----- OK FAILURE!')
        print(f' ----- origin uri: {uri}')


#==============================================================================
# Test Script
#==============================================================================

if __name__ == '__main__':
      
    print('\n *** Test Preparation ***')
    graph = load_test_graph()
    uri_1 = 'net:compositeClass_orbit_hasManner_conjunction-OR'
    uri_2 = '<net:compositeClass_orbit_hasManner_conjunction-OR>'
    uri_3 = 'https://tenet.tetras-libre.fr/semantic-net#atomClass_sun_s2'
    uri_4 = 'https://unknow-url#atomClass_sun_s2'
    uri_5 = URIRef('net:compositeClass_orbit_hasManner_conjunction-OR')
    print('\n \n')
    
    print('\n *** Development Test ***')
    devtest_check_uri(uri_1)
    devtest_check_uri(uri_2)
    devtest_check_uri(uri_3)
    devtest_check_uri(uri_4)
    devtest_check_uri(uri_5)
    devtest_create_uriref(graph, uri_1)
    devtest_create_uriref(graph, uri_1)
    devtest_create_uriref(graph, uri_2)
    devtest_create_uriref(graph, uri_3)
    devtest_create_uriref(graph, uri_4)
    devtest_create_uriref(graph, uri_5)
    print('\n \n')
    
    print('\n *** Unit Test ***')
    unittest_produce_uriref(graph, uri_1)
    unittest_produce_uriref(graph, uri_2)
    unittest_produce_uriref(graph, uri_3)
    unittest_failure_produce_uriref(graph, uri_4)
    unittest_produce_uriref(graph, uri_5)
    unittest_produce_uriref(graph, None)
    print('\n \n')

    print('\n *** - ***')